import config.base
import re
import os

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = ''
    self.substPrefix  = ''
    self.version = {}
    self.rejected = {}
    self.text = ''
    return

  def __str__(self):
    return self.text

  def setupHelp(self, help):
    import nargs
    help.addArgument('Compiler Flags', '-optionsModule=<module name>', nargs.Arg(None, 'config.compilerOptions', 'The Python module used to determine compiler options and versions'))
    help.addArgument('Compiler Flags', '-with-debugging=<bool>', nargs.ArgBool(None, 1, 'Specify debugging version of libraries'))
    help.addArgument('Compiler Flags', '-C_VERSION=<string>',   nargs.Arg(None, 'Unknown', 'The version of the C compiler'))
    help.addArgument('Compiler Flags', '-CXX_VERSION=<string>', nargs.Arg(None, 'Unknown', 'The version of the C++ compiler'))
    help.addArgument('Compiler Flags', '-FC_VERSION=<string>',  nargs.Arg(None, 'Unknown', 'The version of the Fortran compiler'))
    help.addArgument('Compiler Flags', '-CUDA_VERSION=<string>',nargs.Arg(None, 'Unknown', 'The version of the CUDA compiler'))
    help.addArgument('Compiler Flags', '-HIP_VERSION=<string>',nargs.Arg(None, 'Unknown', 'The version of the HIP compiler'))
    help.addArgument('Compiler Flags', '-SYCL_VERSION=<string>',nargs.Arg(None, 'Unknown', 'The version of the SYCL compiler'))
    help.addArgument('Compiler Flags', '-COPTFLAGS=<string>',   nargs.Arg(None, None, 'Override the debugging/optimization flags for the C compiler'))
    help.addArgument('Compiler Flags', '-CXXOPTFLAGS=<string>', nargs.Arg(None, None, 'Override the debugging/optimization flags for the C++ compiler'))
    help.addArgument('Compiler Flags', '-FOPTFLAGS=<string>',   nargs.Arg(None, None, 'Override the debugging/optimization flags for the Fortran compiler'))
    help.addArgument('Compiler Flags', '-CUDAOPTFLAGS=<string>',   nargs.Arg(None, None, 'Override the debugging/optimization flags for the CUDA compiler'))
    help.addArgument('Compiler Flags', '-HIPOPTFLAGS=<string>',   nargs.Arg(None, None, 'Override the debugging/optimization flags for the HIP compiler'))
    help.addArgument('Compiler Flags', '-SYCLOPTFLAGS=<string>',   nargs.Arg(None, None, 'Override the debugging/optimization flags for the SYCL compiler'))
    help.addArgument('Compiler Flags', '-with-gcov=<bool>', nargs.ArgBool(None, value=None, help='Specify that GNUs coverage tool gcov is used', deprecated='--with-coverage'))
    return

  def setupDependencies(self, framework):
    config.base.Configure.setupDependencies(self, framework)
    self.setCompilers = framework.require('config.setCompilers', self)
    return

  def getOptionalFlagsName(self, language, compilerOnly = 0):
    if language == 'C':
      flagsArg = 'COPTFLAGS'
    elif language == 'Cxx':
      if compilerOnly:
        flagsArg = 'CXX_CXXOPTFLAGS'
      else:
        flagsArg = 'CXXOPTFLAGS'
    elif language == 'FC':
      flagsArg = 'FOPTFLAGS'
    elif language == 'CUDA':
      flagsArg = 'CUDAOPTFLAGS'
    elif language == 'HIP':
      flagsArg = 'HIPOPTFLAGS'
    elif language == 'SYCL':
      flagsArg = 'SYCLOPTFLAGS'
    else:
      raise RuntimeError('Unknown language: '+language)
    return flagsArg

  def findOptFlags(self, flags):
    if isinstance(flags, str):
      flags = flags.split()

    return [f for f in flags if f.startswith('-g') or f.startswith('-O') or f in {'-fast'}]

  def hasOptFlags(self,flags):
    return len(self.findOptFlags(flags)) > 0

  def getOptionsObject(self):
    '''Get a configure object which will return default options for each compiler'''
    options = None
    try:
      mod     = __import__(self.argDB['optionsModule'], locals(), globals(), ['CompilerOptions'])
      options = mod.CompilerOptions(self.framework)
      options.setup()
    except ImportError:
      self.logPrint('ERROR: Failed to load user options module '+str(self.argDB['optionsModule']))
    return options

  def configureCompilerFlags(self):
    '''Get the default compiler flags'''
    self.debugging = self.argDB['with-debugging']
    bopts = ['', '+']
    if self.debugging:
      bopts.append('g')
    else:
      bopts.append('O')

    options = self.getOptionsObject()
    if not options:
      return
    options.saveLog()
    for language, compiler in [('C', 'CC'), ('Cxx', 'CXX'), ('FC', 'FC'), ('CUDA', 'CUDAC'), ('HIP', 'HIPC'), ('SYCL', 'SYCLC')]:
      if not hasattr(self.setCompilers, compiler):
        continue
      self.setCompilers.pushLanguage(language)
      flagsName = config.base.Configure.getCompilerFlagsName(language)
      try:
        self.version[language] = self.argDB[language.upper()+'_VERSION']
        if self.version[language] == 'Unknown':
          self.version[language] = options.getCompilerVersion(language, self.setCompilers.getCompiler())
      except RuntimeError:
        pass
      self.rejected[language] = []
      for bopt in bopts:
        userflags = 0
        userlangflags = 0
        if bopt in ['g','O'] and self.getOptionalFlagsName(language) in self.argDB: # check --COPTFLAGS etc
          # treat user supplied options as single option - as it could include options separated by spaces '-tp k8-64'
          flags = [self.argDB[self.getOptionalFlagsName(language)]]
          userflags = 1
        elif bopt in ['g','O'] and self.hasOptFlags(getattr(self.setCompilers,flagsName)): # check --CFLAGS etc
          self.logPrint('Optimization options found in '+flagsName+ '. Skipping setting defaults')
          flags = []
        elif bopt == '' and flagsName in self.argDB:
          self.logPrint('Ignoring default options which were overridden using --'+flagsName+ ' ' + self.argDB[flagsName])
          flags = []
        elif bopt == '+':
          if flagsName+'+' in self.argDB:
            flags = [self.argDB[flagsName+'+']]
            userlangflags = 1
          else:
            continue
        else:
          flags = options.getCompilerFlags(language, self.setCompilers.getCompiler(), bopt)

        for testFlag in flags:
          if isinstance(testFlag, (tuple, list)):
            testFlag = ' '.join(testFlag)
          self.logPrint('Trying '+language+' compiler flag '+testFlag)
          try:
            self.setCompilers.addCompilerFlag(testFlag)
          except RuntimeError:
            if userflags:
              raise RuntimeError('User provided flags for language '+language+' with '+self.getOptionalFlagsName(language)+': '+self.argDB[self.getOptionalFlagsName(language)]+' are not correct for the compiler')
            if userlangflags:
              raise RuntimeError('User provided flags for language '+language+' with '+flagsName+'+: '+self.argDB[flagsName+'+']+' are not correct for the compiler')
            self.logPrint('Rejected '+language+' compiler flag '+testFlag)
            self.rejected[language].append(testFlag)
      self.setCompilers.popLanguage()
    return

  def outputCompilerMacros(self):
    '''Cannot use the regular outputCompile because it swallows the output into the -o file'''
    command = self.getCompilerCmd()
    if self.compilerDefines: self.framework.outputHeader(self.compilerDefines)
    self.framework.outputCHeader(self.compilerFixes)
    f = open(self.compilerSource, 'w')
    f.close()
    try:
      command = command + ' -E -dM '
      start = command.find('-o ')
      end = start + 3 + command[start+3:].find(' ')
      command = command[:start-1]+command[end:]
      (out, err, ret) = Configure.executeShellCommand(command, log = self.log)
      if out.find('__AVX2__') > -1 and out.find('__FMA__') > -1:
        self.text = self.text + 'Intel instruction sets utilizable by compiler:\n'
        self.text = self.text + '  AVX2\n'
      if out.find('__AVX512') > -1:
        self.text = self.text + '  AVX512: '
        self.text = self.text + ' '.join([i for i in out.split('__') if i.startswith('AVX512')])
        self.text = self.text + '\n'
    except:
      pass
    for filename in [self.compilerDefines, self.compilerFixes, self.compilerSource, self.compilerObj]:
      if os.path.isfile(filename): os.remove(filename)

  def checkCompilerMacros(self):
    '''Save the list of CPP macros defined by the C and C++ compiler, does not work for all compilers'''
    '''The values will depends on the flags passed to the compiler'''
    self.outputCompilerMacros()
    if hasattr(self.setCompilers, 'CXX'):
      with self.Language('Cxx'):
        self.outputCompilerMacros()
    return

  def checkIntelHardwareSupport(self):
    '''Use Linux/macOS commands to determine what operations the hardware supports'''
    try:
      (out, err, ret) = Configure.executeShellCommand('lscpu', log = self.log)
    except:
      out = ''
    if out.find(' avx2 ') > -1 and out.find(' fma ') > -1:
      self.text = self.text + 'Intel instruction sets found on CPU:\n'
      self.text = self.text + '  AVX2\n'
    if out.find(' avx512') > -1:
      self.text = self.text + '  AVX512: '
      self.text = self.text + ' '.join([i for i in out.split(' ') if i.startswith('avx512')])
      self.text = self.text + '\n'
    return

  def configure(self):
    self.executeTest(self.configureCompilerFlags)
    self.executeTest(self.checkIntelHardwareSupport)
    self.executeTest(self.checkCompilerMacros)
    return



import config.package
import os

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.gitcommit        = 'v1.3-p2'
    self.download         = ['git://https://bitbucket.org/petsc/pkg-triangle','https://bitbucket.org/petsc/pkg-triangle/get/'+self.gitcommit+'.tar.gz']
    self.downloaddirnames = ['petsc-pkg-triangle','Triangle']
    self.functions        = ['triangulate']
    self.includes         = ['triangle.h']
    self.liblist          = [['libtriangle.a']]
    self.precisions       = ['single', 'double', '__float128']
    self.hastests         = 1
    self.brokengnu23      = 1
    return

  def setupHelp(self, help):
    import nargs
    config.package.Package.setupHelp(self, help)
    help.addArgument('TRIANGLE', '-download-triangle-build-exec=<bool>', nargs.ArgBool(None, False, 'Build and install the triangle executable in addition to the library'))
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.x               = framework.require('config.packages.X', self)
    self.mathlib         = framework.require('config.packages.mathlib',self)
    self.deps            = [self.mathlib]
    return

  def Install(self):
    import os, sys
    import config.base

    libDir         = os.path.join(self.installDir, 'lib')
    includeDir     = os.path.join(self.installDir, 'include')
    makeinc        = os.path.join(self.packageDir, 'make.inc')
    installmakeinc = os.path.join(self.installDir, 'make.inc')

    g = open(makeinc,'w')
    g.write('SHELL            = '+self.programs.SHELL+'\n')
    g.write('CP               = '+self.programs.cp+'\n')
    g.write('RM               = '+self.programs.RM+'\n')
    g.write('MKDIR            = '+self.programs.mkdir+'\n')
    g.write('OMAKE            = '+self.make.make+' '+self.make.noprintdirflag+'\n')

    g.write('CLINKER          = '+self.getLinker()+'\n')
    g.write('AR               = '+self.setCompilers.AR+'\n')
    g.write('ARFLAGS          = '+self.setCompilers.AR_FLAGS+'\n')
    g.write('AR_LIB_SUFFIX    = '+self.setCompilers.AR_LIB_SUFFIX+'\n')
    g.write('RANLIB           = '+self.setCompilers.RANLIB+'\n')
    g.write('SL_LINKER_SUFFIX = '+self.setCompilers.sharedLibraryExt+'\n')

    g.write('TRIANGLE_ROOT    = '+self.packageDir+'\n')
    g.write('PREFIX           = '+self.installDir+'\n')
    g.write('LIBDIR           = '+libDir+'\n')
    g.write('INSTALL_LIB_DIR  = '+libDir+'\n')
    g.write('TRIANGLELIB      = libtriangle.$(AR_LIB_SUFFIX)\n')
    g.write('SHLIB            = libtriangle\n')

    self.pushLanguage('C')
    cflags = self.updatePackageCFlags(self.getCompilerFlags())
    cflags += ' '+self.headers.toString('.')
    cflags += ' -fPIC'
    cflags += ' -DNO_TIMER'

    g.write('CC             = '+self.getCompiler()+'\n')
    g.write('CFLAGS         = '+cflags+'\n')
    self.popLanguage()

    if self.checkSharedLibrariesEnabled():
      import config.setCompilers

      g.write('BUILDSHAREDLIB = yes\n')
      if config.setCompilers.Configure.isSolaris(self.log) and config.setCompilers.Configure.isGNU(self.getCompiler(), self.log):
        g.write('shared_arch: shared_'+sys.platform+'gnu\n')
      else:
        g.write('shared_arch: shared_'+sys.platform+'\n')
        g.write('''
triangle_shared:
	-@if [ "${BUILDSHAREDLIB}" = "no" ]; then \\
	    echo "Shared libraries disabled"; \\
	  else \
	    echo "making shared libraries in ${INSTALL_LIB_DIR}"; \\
	    ${RM} -rf ${INSTALL_LIB_DIR}/tmp-triangle-shlib; \\
	    mkdir ${INSTALL_LIB_DIR}/tmp-triangle-shlib; \\
            cwd=`pwd`; \\
	    for LIBNAME in ${SHLIB}; \\
	    do \\
	      if test -f ${INSTALL_LIB_DIR}/$$LIBNAME.${AR_LIB_SUFFIX} -o -f ${INSTALL_LIB_DIR}/lt_$$LIBNAME.${AR_LIB_SUFFIX}; then \\
	        if test -f ${INSTALL_LIB_DIR}/$$LIBNAME.${SL_LINKER_SUFFIX}; then \\
	          flag=`find ${INSTALL_LIB_DIR} -type f -name $$LIBNAME.${AR_LIB_SUFFIX} -newer ${INSTALL_LIB_DIR}/$$LIBNAME.${SL_LINKER_SUFFIX} -print`; \\
	          if [ "$$flag" = "" ]; then \\
	            flag=`find ${INSTALL_LIB_DIR} -type f -name lt_$$LIBNAME.${AR_LIB_SUFFIX} -newer ${INSTALL_LIB_DIR}/$$LIBNAME.${SL_LINKER_SUFFIX} -print`; \\
	          fi; \\
	        else \\
	          flag="build"; \\
	        fi; \\
	        if [ "$$flag" != "" ]; then \\
                echo "building $$LIBNAME.${SL_LINKER_SUFFIX}"; \\
                  ${RM} ${INSTALL_LIB_DIR}/tmp-triangle-shlib/*; \\
	          cd  ${INSTALL_LIB_DIR}/tmp-triangle-shlib; \\
	          ${AR} x ${INSTALL_LIB_DIR}/$$LIBNAME.${AR_LIB_SUFFIX}; \\
	          ${RANLIB} ${INSTALL_LIB_DIR}/$$LIBNAME.${AR_LIB_SUFFIX}; \\
                  cd $$cwd;\\
	          ${OMAKE} LIBNAME=$$LIBNAME SHARED_LIBRARY_TMPDIR=${INSTALL_LIB_DIR}/tmp-triangle-shlib shared_arch; \\
	        fi; \\
	      fi; \\
	    done; \\
	    ${RM} -rf ${INSTALL_LIB_DIR}/tmp-triangle-shlib; \\
	  fi\n''')
    else:
      g.write('BUILDSHAREDLIB = no\n')
      g.write('shared_arch:\n')
      g.write('shared:\n')
    g.close()

    # Now compile & install
    if self.installNeeded('make.inc'):
      try:
        self.logPrintBox('Compiling Triangle; this may take several minutes')
        output1,err1,ret1  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make clean && make libtriangle.'+self.setCompilers.AR_LIB_SUFFIX+' && make clean', timeout=2500, log = self.log)
      except RuntimeError as e:
        raise RuntimeError('Error running make on Triangle: '+str(e))
      self.logPrintBox('Installing Triangle; this may take several minutes')
      output,err,ret = config.package.Package.executeShellCommand('mkdir -p '+os.path.join(self.installDir,'lib'), timeout=2500, log=self.log)
      output,err,ret = config.package.Package.executeShellCommand('mkdir -p '+os.path.join(self.installDir,'include'), timeout=2500, log=self.log)
      output,err,ret = config.package.Package.executeShellCommand('mkdir -p '+os.path.join(self.installDir,'bin'), timeout=2500, log=self.log)
      output2,err2,ret2  = config.package.Package.executeShellCommand('cp -f '+os.path.join(self.packageDir,'libtriangle.'+self.setCompilers.AR_LIB_SUFFIX)+' '+os.path.join(self.installDir,'lib'), timeout=60, log = self.log)
      output2,err2,ret2  = config.package.Package.executeShellCommand('cp -f '+os.path.join(self.packageDir, 'src', 'triangle.h')+' '+includeDir, timeout=60, log = self.log)
      if self.argDB['download-triangle-build-exec']:
        try:
          output3,err3,ret3  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make clean && '+self.getCompiler()+' '+cflags+' -DNO_PETSC_MALLOC src/triangle.c -o triangle -lm && make clean', timeout=2500, log = self.log)
        except RuntimeError as e:
          raise RuntimeError('Error building Triangle executable: '+str(e))
        output3,err3,ret3  = config.package.Package.executeShellCommand('cp -f '+os.path.join(self.packageDir,'triangle')+' '+os.path.join(self.installDir,'bin'), timeout=60, log = self.log)
      self.postInstall(output1+err1+output2+err2,'make.inc')
    return self.installDir

  def configureLibrary(self):
    config.package.Package.configureLibrary(self)
    return



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--package-prefix-hash='+petsc_hash_pkgs,
    '--with-debugging=0',
    '--with-cuda=1',
    '--with-cudac=clang++',
    '--with-cuda-dialect=17',
    '--with-cc=clang',
    '--with-cxx=clang++',
    '--download-openmpi',
    #'--with-coverage',
  ]

  configure.petsc_configure(configure_options)



#!/usr/bin/env python3
from __future__ import print_function
from __future__ import absolute_import
import project
import RDict

import os
import sys

def getPythonPath():
  if 'PYTHONPATH' in os.environ:
    PYTHONPATH = [p for p in os.environ['PYTHONPATH'].split(os.path.pathsep) if len(p)]
  else:
    PYTHONPATH = []
  argsDB   = RDict.RDict(parentDirectory = os.path.abspath(os.path.dirname(sys.modules['RDict'].__file__)))
  projects = argsDB['installedprojects']
  for p in projects:
    try:
      root = p.getPythonPath()
      for r in root:
        if not r in PYTHONPATH:
          PYTHONPATH.append(r)
    except: pass
  return ':'.join(PYTHONPATH)

if __name__ ==  '__main__':
  if len(sys.argv) > 1: sys.exit('Usage: pythonpath.py')
  print(getPythonPath())




import config.package

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.download      = ['https://web.cels.anl.gov/projects/petsc/download/externalpackages/sprng-1.0.tar.gz']
    self.functions     = ['make_new_seed_mpi']
    self.includes      = ['sprng.h']
    self.liblist       = [['liblcg.a']]
    self.requirekandr  = 1
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.mpi             = framework.require('config.packages.MPI',self)
    self.deps = [self.mpi]
    return

  def Install(self):
    import os

    g = open(os.path.join(self.packageDir,'SRC','make.PETSC'),'w')

    g.write('AR             = '+self.setCompilers.AR+'\n')
    g.write('ARFLAGS        = '+self.setCompilers.AR_FLAGS+'\n')
    g.write('AR_LIB_SUFFIX  = '+self.setCompilers.AR_LIB_SUFFIX+'\n')
    g.write('RANLIB         = '+self.setCompilers.RANLIB+'\n')

    self.pushLanguage('C')
    cflags = self.updatePackageCFlags(self.getCompilerFlags())
    cflags += ' ' + self.headers.toString(self.mpi.include)+' '+self.headers.toString('.')
    cflags += ' ' + '-DSPRNG_MPI' # either using MPI or MPIUNI

    g.write('CC             = '+self.getCompiler()+'\n')
    g.write('CFLAGS         = '+cflags+'\n')
    g.write('CLD            = $(CC)\n')
    g.write('MPICC          = $(CC)\n')
    g.write('CPP            ='+self.getPreprocessor()+'\n')
    self.popLanguage()

    # extra unused options
    g.write('CLDFLAGS       = \n')
    g.write('F77            = echo\n')
    g.write('F77LD          = $(F77)\n')
    g.write('FFXN 	    = -DAdd_\n')
    g.write('FSUFFIX 	    = F\n')
    g.write('MPIF77 	    = echo\n')
    g.write('FFLAGS 	    = \n')
    g.write('F77LDFLAGS     = \n')
    g.close()

    if self.installNeeded(os.path.join('SRC','make.PETSC')):
      try:
        self.logPrintBox('Compiling and installing SPRNG; this may take several minutes')
        output,err,ret = config.package.Package.executeShellCommand('mkdir -p '+os.path.join(self.installDir,'lib'), timeout=2500, log=self.log)
        output,err,ret = config.package.Package.executeShellCommand('mkdir -p '+os.path.join(self.installDir,'include'), timeout=2500, log=self.log)
        output,err,ret  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make realclean && cd SRC && make && cd .. && cp -f lib/*.a '+os.path.join(self.libDir,'')+' && cp -f include/*.h '+os.path.join(self.installDir,self.includedir,''), timeout=2500, log = self.log)
      except RuntimeError as e:
        raise RuntimeError('Error running make on SPRNG: '+str(e))
      self.postInstall(output+err,os.path.join('SRC','make.PETSC'))
    return self.installDir



from __future__ import absolute_import
import logger

import os
from urllib import parse as urlparse_local
import config.base
import socket
import shutil

# Fix parsing for nonstandard schemes
urlparse_local.uses_netloc.extend(['bk', 'ssh', 'svn'])

class Retriever(logger.Logger):
  def __init__(self, sourceControl, clArgs = None, argDB = None):
    logger.Logger.__init__(self, clArgs, argDB)
    self.sourceControl = sourceControl
    self.gitsubmodules = []
    self.gitprereq = 1
    self.git_urls = []
    self.hg_urls = []
    self.dir_urls = []
    self.link_urls = []
    self.tarball_urls = []
    self.stamp = None
    self.ver = 'unknown'
    return

  def isGitURL(self, url):
    parsed = urlparse_local.urlparse(url)
    if (parsed[0] == 'git') or (parsed[0] == 'ssh' and parsed[2].endswith('.git')) or (parsed[0] == 'https' and parsed[2].endswith('.git')):
      return True
    elif os.path.isdir(url) and self.isDirectoryGitRepo(url):
      return True
    return False

  def setupURLs(self,packagename,urls,gitsubmodules,gitprereq):
    self.packagename = packagename
    self.gitsubmodules = gitsubmodules
    self.gitprereq = gitprereq
    for url in urls:
      parsed = urlparse_local.urlparse(url)
      if self.isGitURL(url):
        self.git_urls.append(self.removePrefix(url,'git://'))
      elif parsed[0] == 'hg'or (parsed[0] == 'ssh' and parsed[1].startswith('hg@')):
        self.hg_urls.append(self.removePrefix(url,'hg://'))
      elif parsed[0] == 'dir' or os.path.isdir(url):
        self.dir_urls.append(self.removePrefix(url,'dir://'))
      elif parsed[0] == 'link':
        self.link_urls.append(self.removePrefix(url,'link://'))
      else:
        self.tarball_urls.extend([url])

  def isDirectoryGitRepo(self, directory):
    if not hasattr(self.sourceControl, 'git'):
      self.logPrint('git not found in self.sourceControl - cannot evaluate isDirectoryGitRepo(): '+directory)
      return False
    from config.base import Configure
    for loc in ['.git','']:
      cmd = '%s rev-parse --resolve-git-dir  %s'  % (self.sourceControl.git, os.path.join(directory,loc))
      (output, error, ret) = Configure.executeShellCommand(cmd, checkCommand = Configure.passCheckCommand, log = self.log)
      if not ret:
        return True
    return False

  @staticmethod
  def removeTarget(t):
    if os.path.islink(t) or os.path.isfile(t):
      os.unlink(t) # same as os.remove(t)
    elif os.path.isdir(t):
      shutil.rmtree(t)

  @staticmethod
  def getDownloadFailureMessage(package, url, filename=None):
    slashFilename = '/'+filename if filename else ''
    return '''\
Unable to download package %s from: %s
* If URL specified manually - perhaps there is a typo?
* If your network is disconnected - please reconnect and rerun ./configure
* Or perhaps you have a firewall blocking the download
* You can run with --with-packages-download-dir=/adirectory and ./configure will instruct you what packages to download manually
* or you can download the above URL manually, to /yourselectedlocation%s
  and use the configure option:
  --download-%s=/yourselectedlocation%s
    ''' % (package.upper(), url, slashFilename, package, slashFilename)

  @staticmethod
  def removePrefix(url,prefix):
    '''Replacement for str.removeprefix() supported only since Python 3.9'''
    if url.startswith(prefix):
      return url[len(prefix):]
    return url

  def generateURLs(self):
    if hasattr(self.sourceControl, 'git') and self.gitprereq:
      for url in self.git_urls:
        yield('git',url)
    else:
      self.logPrint('Git not found or gitprereq check failed! skipping giturls: '+str(self.git_urls)+'\n')
    if hasattr(self.sourceControl, 'hg'):
      for url in self.hg_urls:
        yield('hg',url)
    else:
      self.logPrint('Hg not found - skipping hgurls: '+str(self.hg_urls)+'\n')
    for url in self.dir_urls:
      yield('dir',url)
    for url in self.link_urls:
      yield('link',url)
    for url in self.tarball_urls:
      yield('tarball',url)

  def genericRetrieve(self,proto,url,root):
    '''Fetch package from version control repository or tarfile indicated by URL and extract it into root'''
    if proto == 'git':
      return self.gitRetrieve(url,root)
    elif proto == 'hg':
      return self.hgRetrieve(url,root)
    elif proto == 'dir':
      return self.dirRetrieve(url,root)
    elif proto == 'link':
      self.linkRetrieve(url,root)
    elif proto == 'tarball':
      self.tarballRetrieve(url,root)

  def dirRetrieve(self, url, root):
    self.logPrint('Retrieving %s as directory' % url, 3, 'install')
    if not os.path.isdir(url): raise RuntimeError('URL %s is not a directory' % url)

    t = os.path.join(root,os.path.basename(url))
    self.removeTarget(t)
    shutil.copytree(url,t)

  def linkRetrieve(self, url, root):
    self.logPrint('Retrieving %s as link' % url, 3, 'install')
    if not os.path.isdir(url): raise RuntimeError('URL %s is not pointing to a directory' % url)

    t = os.path.join(root,os.path.basename(url))
    self.removeTarget(t)
    os.symlink(os.path.abspath(url),t)

  def gitRetrieve(self, url, root):
    self.logPrint('Retrieving %s as git repo' % url, 3, 'install')
    if not hasattr(self.sourceControl, 'git'):
      raise RuntimeError('self.sourceControl.git not set')
    if os.path.isdir(url) and not self.isDirectoryGitRepo(url):
      raise RuntimeError('URL %s is a directory but not a git repository' % url)

    newgitrepo = os.path.join(root,'git.'+self.packagename)
    self.removeTarget(newgitrepo)

    try:
      submodopt =''
      for itm in self.gitsubmodules:
        submodopt += ' --recurse-submodules='+itm
      config.base.Configure.executeShellCommand('%s clone %s %s %s' % (self.sourceControl.git, submodopt, url, newgitrepo), log = self.log, timeout = 120.0)
    except  RuntimeError as e:
      self.logPrint('ERROR: '+str(e))
      failureMessage = self.getDownloadFailureMessage(self.packagename, url)
      raise RuntimeError('Unable to clone '+self.packagename+'\n'+str(e)+'\n'+failureMessage)

  def hgRetrieve(self, url, root):
    self.logPrint('Retrieving %s as hg repo' % url, 3, 'install')
    if not hasattr(self.sourceControl, 'hg'):
      raise RuntimeError('self.sourceControl.hg not set')

    newgitrepo = os.path.join(root,'hg.'+self.packagename)
    self.removeTarget(newgitrepo)
    try:
      config.base.Configure.executeShellCommand('%s clone %s %s' % (self.sourceControl.hg, url, newgitrepo), log = self.log, timeout = 120.0)
    except  RuntimeError as e:
      self.logPrint('ERROR: '+str(e))
      failureMessage = self.getDownloadFailureMessage(self.packagename, url)
      raise RuntimeError('Unable to clone '+self.packagename+'\n'+str(e)+'\n'+failureMessage)

  def tarballRetrieve(self, url, root):
    parsed = urlparse_local.urlparse(url)
    filename = os.path.basename(parsed[2])
    localFile = os.path.join(root,'_d_'+filename)
    self.logPrint('Retrieving %s as tarball to %s' % (url,localFile) , 3, 'install')
    ext =  os.path.splitext(localFile)[1]
    if ext not in ['.bz2','.tbz','.gz','.tgz','.zip','.ZIP']:
      raise RuntimeError('Unknown compression type in URL: '+ url)

    self.removeTarget(localFile)

    if parsed[0] == 'file' and not parsed[1]:
      url = parsed[2]
    if os.path.exists(url):
      if not os.path.isfile(url):
        raise RuntimeError('Local path exists but is not a regular file: '+ url)
      # copy local file
      shutil.copyfile(url, localFile)
    else:
      # fetch remote file
      try:
        from urllib.request import Request, urlopen
        sav_timeout = socket.getdefaulttimeout()
        socket.setdefaulttimeout(30)
        req = Request(url)
        req.headers['User-Agent'] = 'PetscConfigure/'+self.ver
        with open(localFile, 'wb') as f:
          f.write(urlopen(req).read())
        socket.setdefaulttimeout(sav_timeout)
      except Exception as e:
        socket.setdefaulttimeout(sav_timeout)
        failureMessage = self.getDownloadFailureMessage(self.packagename, url, filename)
        raise RuntimeError(str(e)+'\n'+failureMessage)

    self.logPrint('Extracting '+localFile)
    if ext in ['.zip','.ZIP']:
      config.base.Configure.executeShellCommand('cd '+root+'; unzip '+localFile, log = self.log)
      output = config.base.Configure.executeShellCommand('cd '+root+'; zipinfo -1 '+localFile+' | head -n 1', log = self.log)
      dirname = os.path.normpath(output[0].strip())
    else:
      failureMessage = '''\
Downloaded package %s from: %s is not a tarball.
[or installed python cannot process compressed files]
* If you are behind a firewall - please fix your proxy and rerun ./configure
  For example at LANL you may need to set the environmental variable http_proxy (or HTTP_PROXY?) to  http://proxyout.lanl.gov
* You can run with --with-packages-download-dir=/adirectory and ./configure will instruct you what packages to download manually
* or you can download the above URL manually, to /yourselectedlocation/%s
  and use the configure option:
  --download-%s=/yourselectedlocation/%s
''' % (self.packagename.upper(), url, filename, self.packagename, filename)
      import tarfile
      try:
        tf  = tarfile.open(os.path.join(root, localFile))
      except tarfile.ReadError as e:
        raise RuntimeError(str(e)+'\n'+failureMessage)
      if not tf: raise RuntimeError(failureMessage)
      #git puts 'pax_global_header' as the first entry and some tar utils process this as a file
      firstname = tf.getnames()[0]
      if firstname == 'pax_global_header':
        firstmember = tf.getmembers()[1]
      else:
        firstmember = tf.getmembers()[0]
      # some tarfiles list packagename/ but some list packagename/filename in the first entry
      if firstmember.isdir():
        dirname = firstmember.name
      else:
        dirname = os.path.dirname(firstmember.name)
      tf.extractall(root)
      tf.close()

    # fix file permissions for the untared tarballs.
    try:
      # check if 'dirname' is set'
      if dirname:
        config.base.Configure.executeShellCommand('cd '+root+'; chmod -R a+r '+dirname+';find  '+dirname + r' -type d -name "*" -exec chmod a+rx {} \;', log = self.log)
      else:
        self.logPrintBox('WARNING: Could not determine dirname extracted by '+localFile+' to fix file permissions')
    except RuntimeError as e:
      raise RuntimeError('Error changing permissions for '+dirname+' obtained from '+localFile+ ' : '+str(e))
    os.unlink(localFile)



import config.package

class Configure(config.package.CMakePackage):
  def __init__(self, framework):
    config.package.CMakePackage.__init__(self, framework)
    self.gitcommit       = 'f136d2a3ac89b3203affef1f04da6fde9d60bf7e'
    self.download        = ['git://https://github.com/bldenton/oce.git','https://github.com/bldenton/oce/archive/'+self.gitcommit+'.tar.gz']
    self.version         = '7.5.0'
    self.minversion      = '7.5.0'
    self.versionname     = 'OCC_VERSION_COMPLETE'
    self.requiresversion = 1
    self.functions       = []
    self.includes        = ['opencascade/Standard_Version.hxx']
    self.liblist         = [['libTKXSBase.a', 'libTKSTEPBase.a', 'libTKSTEPAttr.a', 'libTKSTEP209.a', 'libTKSTEP.a', 'libTKIGES.a', 'libTKGeomAlgo.a', 'libTKTopAlgo.a', 'libTKPrim.a', 'libTKBO.a', 'libTKBool.a', 'libTKHLR.a', 'libTKFillet.a', 'libTKOffset.a', 'libTKFeat.a', 'libTKMesh.a', 'libTKXMesh.a', 'libTKShHealing.a', 'libTKG2d.a', 'libTKG3d.a', 'libTKGeomBase.a', 'libTKBRep.a', 'libTKernel.a', 'libTKMath.a']]
    self.pkgname         = 'opencascade'
    self.buildLanguages  = ['Cxx']
    self.hastests        = 1
    self.need35policy    = True
    return

  def setupDependencies(self, framework):
    config.package.CMakePackage.setupDependencies(self, framework)
    self.compilerFlags = framework.require('config.compilerFlags', self)
    self.deps          = []
    return

  def formCMakeConfigureArgs(self):
    args = config.package.CMakePackage.formCMakeConfigureArgs(self)
    if not hasattr(self.compilers, 'CXX'):
      raise RuntimeError("%s requires a C++ compiler\n" % self.pkgname)
    return args

  def updateControlFiles(self):
    import os.path
    # The OCE build relies on files in the adm directory
    controlDir = os.path.join(self.packageDir, 'adm')
    modfile = os.path.join(controlDir, 'MODULES')
    fd = open(modfile, 'w')
    fd.write('''\
FoundationClasses TKernel TKMath
ModelingData TKG2d TKG3d TKGeomBase TKBRep
ModelingAlgorithms TKGeomAlgo TKTopAlgo TKPrim TKBO TKBool TKHLR TKFillet TKOffset TKFeat TKMesh TKXMesh TKShHealing
Visualization
ApplicationFramework
DataExchange TKXSBase TKSTEPBase TKSTEPAttr TKSTEP209 TKSTEP TKIGES
Draw
''')
    fd.close()
    toolfile = os.path.join(controlDir, 'TOOLS')
    fd = open(toolfile, 'w')
    fd.write('')
    fd.close()
    sampfile = os.path.join(controlDir, 'SAMPLES')
    fd = open(sampfile, 'w')
    fd.write('')
    fd.close()
    return



import args
import config.compile.processor
import config.compile.C
import config.framework
import config.libraries
import os
import sys

import config.setsOrdered as sets

'''
HIP is a C++ compiler with extensions to support the HIP programming model.
Because of its slowness, and in some ways the extensions make it a new language,
we have a separate compiler for it.
But we use the extension .hip.cpp to denote these files similar to what is done
for SYCL and following the recommendations of AMD
'''
class Preprocessor(config.compile.processor.Processor):
  '''The HIP preprocessor'''
  def __init__(self, argDB):
    config.compile.processor.Processor.__init__(self, argDB, 'HIPPP', 'HIPPPFLAGS', '.hip.cpp', '.hip.cpp')
    self.language        = 'HIP'
    self.includeDirectories = sets.Set()
    return

class Compiler(config.compile.processor.Processor):
  '''The HIP compiler'''
  def __init__(self, argDB, usePreprocessorFlags = True):
    config.compile.processor.Processor.__init__(self, argDB, 'HIPC', 'HIPFLAGS', '.hip.cpp', '.o')
    self.language        = 'HIP'
    self.requiredFlags[-1]  = '-c'
    self.outputFlag         = '-o'
    self.includeDirectories = sets.Set()
    if usePreprocessorFlags:
      self.flagsName.extend(Preprocessor(argDB).flagsName)

    return

  def getTarget(self, source):
    '''Return None for header files'''
    import os

    # HIP files are foo.hip.cpp
    base1, ext1 = os.path.splitext(source)
    base2, ext2 = os.path.splitext(base1)
    if ext1 == '.h':
      return None
    # If there is no .hip, then not a hip file
    if not ext2:
        return None
    return base2+'.o'

  def getCommand(self, sourceFiles, outputFile = None):
    '''If no outputFile is given, do not execute anything'''
    if outputFile is None:
      return 'true'
    return config.compile.processor.Processor.getCommand(self, sourceFiles, outputFile)

class Linker(config.compile.C.Linker):
  '''The HIP linker'''
  def __init__(self, argDB):
    self.compiler        = Compiler(argDB, usePreprocessorFlags = False)
    self.configLibraries = config.libraries.Configure(config.framework.Framework(clArgs = '', argDB = argDB, tmpDir = os.getcwd()))
    config.compile.processor.Processor.__init__(self, argDB, [self.compiler.name], ['HIPC_LINKER_FLAGS'], '.o', '.a')
    self.language   = 'HIP'
    self.outputFlag = '-o'
    self.libraries  = sets.Set()
    return

  def getExtraArguments(self):
    if not hasattr(self, '_extraArguments'):
      return ''
    return self._extraArguments
  extraArguments = property(getExtraArguments, config.compile.processor.Processor.setExtraArguments, doc = 'Optional arguments for the end of the command')

class SharedLinker(config.compile.C.SharedLinker):
  '''The HIP shared linker: Just use Cxx linker for now'''
  def __init__(self, argDB):
    config.compile.Cxx.SharedLinker.__init__(self, argDB)
    self.language = 'HIP'
    return

class StaticLinker(config.compile.C.StaticLinker):
  '''The HIP static linker, just use Cxx for now'''
  def __init__(self, argDB):
    config.compile.Cxx.StaticLinker.__init__(self, argDB)
    self.language = 'HIP'
    return

class DynamicLinker(config.compile.C.DynamicLinker):
  '''The HIP dynamic linker, just use Cxx for now'''
  def __init__(self, argDB):
    config.compile.Cxx.DynamicLinker.__init__(self, argDB)
    self.language = 'HIP'
    return



#!/usr/bin/env python3
"""
This module holds configuration information for the petsc_tas_analysis.py in
  a dictionary format.  This module takes no parameters.

The dict filePath holds key value pairs for file paths:
  The key absoluteGraphs has contains the string of the absolute path to place
  the generated graphs.

The key absoluteData has contains the string of the absolute path to look
  for the file containing the information to be analyzed.  Within petsc_tas_analysis
  the system.path variable is appended to include this path.

The dict fieldNames holds key value pairs for the fields of problems.  This is
  values for the level are also dicts since many problems have multiple fields.
  The key ex62 contains the name of fields for example 62, ie ex62.c
"""

filePath = {}
filePath['defaultData']=''
filePath['defaultGraphs']=''

fieldNames = {}
fieldNames['ex62'] = {}
fieldNames['ex76'] = {}
fieldNames['ex62']['field 0'] = 'Velocity'
fieldNames['ex62']['field 1'] = 'Pressure'
fieldNames['ex76']['field 0'] = 'Velocity'
fieldNames['ex76']['field 1'] = 'Pressure'
fieldNames['ex76']['field 2'] = 'Temperature'


from __future__ import print_function
try:
  import readline
except ImportError: pass

class Arg(object):
  '''This is the base class for all objects contained in RDict. Access to the raw argument values is
provided by getValue() and setValue(). These objects can be thought of as type objects for the
values themselves. It is possible to set an Arg in the RDict which has not yet been assigned a value
in order to declare the type of that option.

Inputs which cannot be converted to the correct type will cause TypeError, those failing validation
tests will cause ValueError.
'''
  def __init__(self, key, value = None, help = '', isTemporary = False, deprecated = False):
    self.key         = key
    self.help        = help
    self.isTemporary = isTemporary
    self.deprecated  = False
    if not value is None:
      self.setValue(value)
    self.deprecated  = deprecated
    return

  def isValueSet(self):
    '''Determines whether the value of this argument has been set'''
    return hasattr(self, 'value')

  def getTemporary(self):
    '''Retrieve the flag indicating whether the item should be persistent'''
    return self.isTemporary

  def setTemporary(self, isTemporary):
    '''Set the flag indicating whether the item should be persistent'''
    self.isTemporary = isTemporary
    return

  def parseValue(arg):
    '''Return the object represented by the value portion of a string argument'''
    # Should I replace this with a lexer?
    if arg: arg = arg.strip()
    if arg and arg[0] == '[' and arg[-1] == ']':
      if len(arg) > 2: value = arg[1:-1].split(',')
      else:            value = []
    elif arg and arg[0] == '{' and arg[-1] == '}':
      value = {}
      idx = 1
      oldIdx = idx
      while idx < len(arg)-1:
        if arg[oldIdx] == ',':
          oldIdx += 1
        while not arg[idx] == ':': idx += 1
        key = arg[oldIdx:idx]
        idx += 1
        oldIdx = idx
        nesting = 0
        while not (arg[idx] == ',' or arg[idx] == '}') or nesting:
          if arg[idx] == '[':
            nesting += 1
          elif arg[idx] == ']':
            nesting -= 1
          idx += 1
        value[key] = Arg.parseValue(arg[oldIdx:idx])
        oldIdx = idx
    else:
      value = arg
    return value
  parseValue = staticmethod(parseValue)

  def parseArgument(arg, ignoreDouble = 0):
    '''Split an argument into a (key, value) tuple, stripping off the leading dashes. Return (None, None) on failure.'''
    start = 0
    if arg and arg[0] == '-':
      start = 1
      if arg[1] == '-' and not ignoreDouble:
        start = 2
    if arg.find('=') >= 0:
      (key, value) = arg[start:].split('=', 1)
    else:
      if start == 0:
        (key, value) = (None, arg)
      else:
        (key, value) = (arg[start:], '1')
    return (key, Arg.parseValue(value))

  parseArgument = staticmethod(parseArgument)

  def findArgument(key, argList):
    '''Locate an argument with the given key in argList, returning the value or None on failure
       - This is generally used to process arguments which must take effect before canonical argument parsing'''
    if not isinstance(argList, list): return None
    # Reverse the list so that we preserve the semantics which state that the last
    #   argument with a given key takes effect
    l = argList[:]
    l.reverse()
    for arg in l:
      (k, value) = Arg.parseArgument(arg)
      if k == key:
        return value
    return None
  findArgument = staticmethod(findArgument)

  def processAlternatePrefixes(argList):
    '''Convert alternate prefixes to our normal form'''
    for l in range(0, len(argList)):
      name = argList[l]
      if name.find('enable-') >= 0:
        argList[l] = name.replace('enable-','with-')
        if name.find('=') == -1: argList[l] = argList[l]+'=1'
      if name.find('disable-') >= 0:
        argList[l] = name.replace('disable-','with-')
        if name.find('=') == -1: argList[l] = argList[l]+'=0'
        elif name.endswith('=1'): argList[l].replace('=1','=0')
      if name.find('without-') >= 0:
        argList[l] = name.replace('without-','with-')
        if name.find('=') == -1: argList[l] = argList[l]+'=0'
        elif name.endswith('=1'): argList[l].replace('=1','=0')
    return
  processAlternatePrefixes = staticmethod(processAlternatePrefixes)

  def __str__(self):
    if not self.isValueSet():
      return 'Empty '+str(self.__class__)
    value = self.value
    if isinstance(value, list):
      return str(list(map(str, value)))
    return str(value)

  def getKey(self):
    '''Returns the key. SHOULD MAKE THIS A PROPERTY'''
    return self.key

  def setKey(self, key):
    '''Set the key. SHOULD MAKE THIS A PROPERTY'''
    self.key = key
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      raise KeyError('Could not find value for key '+str(self.key))
    return self.value

  def checkKey(self):
    if self.deprecated:
      if isinstance(self.deprecated, str):
        raise KeyError('Deprecated option '+self.key+' should be '+self.deprecated)
      raise KeyError('Deprecated option '+self.key)
    return

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    self.value = value
    return

class ArgBool(Arg):
  '''Arguments that represent boolean values'''
  def __init__(self, key, value = None, help = '', isTemporary = 0, deprecated = False):
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    try:
      if   value == 'no':    value = 0
      elif value == 'yes':   value = 1
      elif value == 'true':  value = 1
      elif value == 'false': value = 0
      elif value == 'True':  value = 1
      elif value == 'False': value = 0
      else:                  value = int(value)
    except:
      raise TypeError('Invalid boolean value: '+str(value)+' for key '+str(self.key))
    self.value = value
    return

class ArgFuzzyBool(Arg):
  '''Arguments that represent boolean values of an extended set'''
  def __init__(self, key, value = None, help = '', isTemporary = 0, deprecated = False):
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def valueName(self, value):
    if value == 0:
      return 'no'
    elif value == 1:
      return 'yes'
    elif value == 2:
      return 'ifneeded'
    return str(value)

  def __str__(self):
    if not self.isValueSet():
      return 'Empty '+str(self.__class__)
    elif isinstance(self.value, list):
      return str(map(self.valueName, self.value))
    return self.valueName(self.value)

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    try:
      if   value == '0':        value = 0
      elif value == '1':        value = 1
      elif value == 'no':       value = 0
      elif value == 'yes':      value = 1
      elif value == 'false':    value = 0
      elif value == 'true':     value = 1
      elif value == 'maybe':    value = 2
      elif value == 'ifneeded': value = 2
      elif value == 'client':   value = 2
      elif value == 'server':   value = 3
      else:                     value = int(value)
    except:
      raise TypeError('Invalid fuzzy boolean value: '+str(value)+' for key '+str(self.key))
    self.value = value
    return

class ArgInt(Arg):
  '''Arguments that represent integer numbers'''
  def __init__(self, key, value = None, help = '', min = -2147483647, max = 2147483648, isTemporary = 0, deprecated = False):
    self.min = min
    self.max = max
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    try:
      value = int(value)
    except:
      raise TypeError('Invalid integer number: '+str(value)+' for key '+str(self.key))
    if value < self.min or value >= self.max:
      raise ValueError('Number out of range: '+str(value)+' not in ['+str(self.min)+','+str(self.max)+')'+' for key '+str(self.key))
    self.value = value
    return

class ArgReal(Arg):
  '''Arguments that represent floating point numbers'''
  def __init__(self, key, value = None, help = '', min = -1.7976931348623157e308, max = 1.7976931348623157e308, isTemporary = 0, deprecated = False):
    self.min = min
    self.max = max
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    try:
      value = float(value)
    except:
      raise TypeError('Invalid floating point number: '+str(value)+' for key '+str(self.key))
    if value < self.min or value >= self.max:
      raise ValueError('Number out of range: '+str(value)+' not in ['+str(self.min)+','+str(self.max)+')'+' for key '+str(self.key))
    self.value = value
    return

class ArgDir(Arg):
  '''Arguments that represent directories'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    # Should check whether it is a well-formed path
    if not isinstance(value, str):
      raise TypeError('Invalid directory: '+str(value)+' for key '+str(self.key))
    value = os.path.expanduser(value)
    value = os.path.abspath(value)
    if self.mustExist and value and not os.path.isdir(value):
      raise ValueError('Nonexistent directory: '+str(value)+' for key '+str(self.key))
    self.value = value
    return

class ArgDirList(Arg):
  '''Arguments that represent directory lists'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    if not isinstance(value, list):
      value = value.split(':')
    # Should check whether it is a well-formed path
    nvalue = []
    for dir in value:
      if dir:
        nvalue.append(os.path.expanduser(dir))
    value = nvalue
    for dir in value:
      if self.mustExist and not os.path.isdir(dir):
        raise ValueError('Invalid directory: '+str(dir)+' for key '+str(self.key))
    self.value = value
    return

class ArgFile(Arg):
  '''Arguments that represent a file'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    # Should check whether it is a well-formed path
    if not isinstance(value, str):
      raise TypeError('Invalid file: '+str(value)+' for key '+str(self.key))
    value = os.path.expanduser(value)
    value = os.path.abspath(value)
    if self.mustExist and value and not os.path.isfile(value):
      raise ValueError('Nonexistent file: '+str(value)+' for key '+str(self.key))
    self.value = value
    return

class ArgFileList(Arg):
  '''Arguments that represent file lists'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    if not isinstance(value, list):
      value = value.split(':')
    # Should check whether it is a well-formed path
    nvalue = []
    for file in value:
      if file:
        nvalue.append(os.path.expanduser(file))
    value = nvalue
    for file in value:
      if self.mustExist and not os.path.isfile(file):
        raise ValueError('Invalid file: '+str(file)+' for key '+str(self.key))
    self.value = value
    return

class ArgLibrary(Arg):
  '''Arguments that represent libraries'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    # Should check whether it is a well-formed path and an archive or shared object
    if self.mustExist:
      if not isinstance(value, list):
        value = value.split(' ')
    self.value = value
    return

class ArgExecutable(Arg):
  '''Arguments that represent executables'''
  def __init__(self, key, value = None, help = '', mustExist = 1, isTemporary = 0, deprecated = False):
    self.mustExist = mustExist
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def getValue(self):
    '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
    if not self.isValueSet():
      return Arg.getValue(self)
    return self.value

  def checkExecutable(self, dir, name):
    import os
    prog = os.path.join(dir, name)
    return os.path.isfile(prog) and os.access(prog, os.X_OK)

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    # Should check whether it is a well-formed path
    if self.mustExist:
      index = value.find(' ')
      if index >= 0:
        options = value[index:]
        value   = value[:index]
      else:
        options = ''
      found = self.checkExecutable('', value)
      if not found:
        for dir in os.environ['PATH'].split(os.path.pathsep):
          if self.checkExecutable(dir, value):
            found = 1
            break
      if not found:
        raise ValueError('Invalid executable: '+str(value)+' for key '+str(self.key))
      value += options
    self.value = value
    return

class ArgString(Arg):
  '''Arguments that represent strings satisfying a given regular expression'''
  def __init__(self, key, value = None, help = '', regExp = None, isTemporary = 0, deprecated = False):
    self.regExp = regExp
    if self.regExp:
      import re
      self.re = re.compile(self.regExp)
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    self.checkKey()
    if self.regExp and not self.re.match(value):
      raise ValueError('Invalid string '+str(value)+'. You must give a string satisfying "'+str(self.regExp)+'"'+' for key '+str(self.key))
    self.value = value
    return

class ArgDownload(Arg):
  '''Arguments that represent software downloads'''
  def __init__(self, key, value = None, help = '', isTemporary = 0, deprecated = False):
    Arg.__init__(self, key, value, help, isTemporary, deprecated)
    return

  def valueName(self, value):
    if value == 0:
      return 'no'
    elif value == 1:
      return 'yes'
    return str(value)

  def __str__(self):
    if not self.isValueSet():
      return 'Empty '+str(self.__class__)
    elif isinstance(self.value, list):
      return str(map(self.valueName, self.value))
    return self.valueName(self.value)

  def setValue(self, value):
    '''Set the value. SHOULD MAKE THIS A PROPERTY'''
    import os
    self.checkKey()
    try:
      if   value == '0':        value = 0
      elif value == '1':        value = 1
      elif value == 'no':       value = 0
      elif value == 'yes':      value = 1
      elif value == 'false':    value = 0
      elif value == 'true':     value = 1
      elif not isinstance(value, int):
        value = str(value)
    except:
      raise TypeError('Invalid download value: '+str(value)+' for key '+str(self.key))
    if isinstance(value, str) and value.startswith('git@'):
      # git@github.com:xrq-phys/blis_apple.git -> git://https://github.com/xrq-phys/blis_apple.git
      value = 'git://https://'+(value[len('git@'):].replace(':','/'))
    if isinstance(value, str):
      from urllib import parse as urlparse_local
      if not urlparse_local.urlparse(value)[0] and not os.path.exists(value):
        raise ValueError('Invalid download location: '+str(value)+' for key '+str(self.key))
    self.value = value
    return



import config.package
import os

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.functions        = ['regexec', 'regcomp', 'regfree']
    self.includes         = ['regex.h']
    self.liblist          = [['libregex.a']]
    self.lookforbydefault = 1
    return



# Stateless utilities
#
# This entire module is exported into the config.* namespace (see
# __init__.py), so be careful to avoid namespace conflicts.  Do not
# import config.util directly.  Instead, import config and access only
# the exported objects.

def classify(items, functional, args=(), kwargs=dict()):
    '''Classify items as True or False using boolean functional on sets.

    An item can only be identified as False if functional([item]) is
    false, but an item is True if functional(List) is True and item in
    List.

    Functional may return True (all items are True), False (at least one
    item is False) or a list of suggestions for items that may have been
    False (implies that at least one item is False).  The list of
    suggestions does not have to be accurate, but classification will be
    faster if it is accurate.
    '''
    items = list(items)         # In case a set or other iterable was passed in

    result = functional(items, *args, **kwargs)
    if result is True:
        return items, []        # All succeeded
    if len(items) == 1:
        return [], items        # Sole failure
    if result is False:
        suggested = []
    else:
        suggested = list(result)
    items = [i for i in items if i not in suggested]
    good = []
    bad = []
    if len(items) < 5:          # linear check
        groups = [[i] for i in items]
    else:                       # bisect
        groups = [items[:len(items)//2], items[len(items)//2:]]
    groups += [[i] for i in suggested]
    for grp in groups:
        g, b = classify(grp, functional, args, kwargs)
        good += g
        bad += b
    return good, bad


class NamedInStderr:
    '''Hepler class to log the (string) items that are written to stderr on failure.

    In the common case, all the missing items are named in the linker
    error and the rest can be confirmed True in a single batch.
    '''
    def __init__(self, items):
        self.named = []
        self.items = items

    def examineStderr(self, ret, out, err):
        if ret:
            self.named += [i for i in self.items if i in err]


class memoize(dict):
    '''Memoizing decorator.  No support for keyword arguments.'''
    def __init__(self, func):
        self.func = func

    def __call__(self, *args):
        try:
            return self[args]
        except KeyError:
            ret = self[args] = self.func(*args)
            return ret



import config.package

class Configure(config.package.GNUPackage):
  def __init__(self, framework):
    config.package.GNUPackage.__init__(self, framework)
    self.gitcommit         = 'de85222e71148d2ae47e6af3f9e48dc98323cffb' # master Aug 4, 2021
    self.download          = ['git://https://github.com/ornladios/ADIOS.git', 'https://github.com/ornladios/ADIOS/archive/'+self.gitcommit+'.tar.gz']
    self.downloaddirnames  = ['adios']
    self.functions         = ['adios_open']
    self.includes          = ['adios.h']
    self.liblist           = [['libadiosf.a', 'libadios.a'],['libadios.a']]
    self.hastests          = 1
    return

  def setupDependencies(self, framework):
    config.package.GNUPackage.setupDependencies(self, framework)
    self.zlib           = framework.require('config.packages.zlib',self)
    self.mpi            = framework.require('config.packages.MPI', self)
    self.hdf5           = framework.require('config.packages.hdf5', self)
    self.netcdf         = framework.require('config.packages.netcdf', self)
    self.pthread        = framework.require('config.packages.pthread', self)
    self.mathlib        = framework.require('config.packages.mathlib',self)
    self.deps           = [self.mpi,self.pthread,self.mathlib]
    self.odeps          = [self.hdf5,self.netcdf,self.zlib]
    return

  def gitPreReqCheck(self):
    '''ADIOS from the git repository needs the GNU autotools'''
    return self.programs.autoreconf and self.programs.libtoolize

  def formGNUConfigureArgs(self):
    '''Add ADIOS specific configure arguments'''
    args = config.package.GNUPackage.formGNUConfigureArgs(self)
    self.pushLanguage('C')
    args.append('MPICC="'+self.getCompiler()+'"')
    self.popLanguage()
    if hasattr(self.compilers, 'CXX'):
      self.pushLanguage('Cxx')
      args.append('MPICXX="'+self.getCompiler()+'"')
      self.popLanguage()
    if hasattr(self.compilers, 'FC'):
      self.pushLanguage('FC')
      args.append('MPIFC="'+self.getCompiler()+'"')
      self.popLanguage()
    else:
      args.append('--disable-fortran')
    if self.hdf5.found:
      args.append('--with-phdf5=yes')
      args.append('--with-phdf5-incdir='+self.installDir)
      args.append('--with-phdf5-libdir='+self.installDir)
      args.append('--with-phdf5-libs=" "')
    if self.netcdf.found:
      args.append('--with-nc4par=yes')
      args.append('--with-nc4par-incdir='+self.installDir)
      args.append('--with-nc4par-libdir='+self.installDir)
      args.append('--with-nc4par-libs=" "')
      args.append('NETCDF_LIBS=" "')
    if self.zlib.found:
      args.append('--with-zlib=yes')
      args.append('--with-zlib-incdir='+self.installDir)
      args.append('--with-zlib-libdir='+self.installDir)
      args.append('--with-zlib-libs=" "')
      args.append('ZLIB_LIBS=" "')

    args.append('CPPFLAGS="'+self.headers.toStringNoDupes(self.dinclude)+'"')
    args.append('LIBS="'+self.libraries.toStringNoDupes(self.dlib)+'"')

    return args




#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

configure_options = [
  '--package-prefix-hash='+petsc_hash_pkgs,
  '--with-clanguage=cxx',
  '--with-debugging=0',

  #'--useThreads=0', # for some reason cmake hangs when invoked from configure on bsd?
  #'-download-fblaslapack=1',
  '--download-mpich=1',
  '--download-mpich-device=ch3:sock',
  '--download-cmake=1',
  '--download-metis=1',
  '--download-parmetis=1',
  '--download-triangle=1',
  '--download-triangle-build-exec=1',
  '--download-superlu=1',
  '--download-superlu_dist=1',
  '--download-scalapack=1',
  '--download-mumps=1',
  '--download-elemental=1',
  '--download-hdf5',
  '--with-zlib=1',
  '--download-sundials2=1',
  '--download-hypre=1',
  '--download-suitesparse=1',
  '--download-make=1', # required by suitesparse
  '--download-chaco=1',
  '--download-spai=1',
  '--download-netcdf=1',
  #'--download-moab=1', # moab-5.5: OS freebsd12.1 is not supported # if needed, use https://web.cels.anl.gov/projects/petsc/download/externalpackages/moab-5.0.0.tar.gz
  '--download-saws',
  '--download-ks',
  '--download-codipack=1',
  '--download-adblaslapack=1',
  '--download-hpddm=1',
  '--with-strict-petscerrorcode',
  ]

if __name__ == '__main__':
  import sys,os
  sys.path.insert(0,os.path.abspath('config'))
  import configure
  configure.petsc_configure(configure_options)



from __future__ import generators
import config.base
import os
import re

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = 'PETSC'
    self.substPrefix  = 'PETSC'
    self.updated      = 0
    self.strmsg       = ''
    return

  def __str__(self):
    return self.strmsg

  def setupHelp(self, help):
    import nargs
    help.addArgument('PETSc', '-with-proc-filesystem=<bool>', nargs.ArgBool(None, 1, 'Use the /proc filesystem for system statistics'))
    return

  def setupDependencies(self, framework):
    config.base.Configure.setupDependencies(self, framework)
    self.functions = framework.require('config.functions', self)
    self.functions.functions.append('getrusage')
    self.functions.functions.append('sbreak')
    self.functions.functions.append('getpagesize')
    return

  def configureMemorySize(self):
    '''Try to determine how to measure the memory usage'''

    # sbreak() is used on cray t3d/e
    if self.functions.haveFunction('sbreak'):
      self.addDefine('USE_SBREAK_FOR_SIZE', 1)
      return

    # /proc is used on Linux systems
    if self.argDB['with-proc-filesystem'] and not self.argDB['with-batch']:
      if os.path.isfile(os.path.join('/proc',str(os.getpid()),'statm')):
        self.addDefine('USE_PROC_FOR_SIZE', 1)
        try:
          fd = open(os.path.join('/proc',str(os.getpid()),'statm'))
          l  = fd.readline().split(' ')
          # make sure we can access the rss field
          if not l[1].isdigit():
            raise RuntimeError("/proc stat file has wrong format rss not integer:"+l[1])
          self.logPrint("Using /proc for PetscMemoryGetCurrentUsage()")
          return
        except:
          pass

    # getrusage() is still used on BSD systems
    if self.functions.haveFunction('getrusage') and not self.argDB['with-batch']:
      if self.functions.haveFunction('getpagesize'):
        (output,status) = self.outputRun('''#include <stdio.h>\n#include <ctype.h>\n#include <sys/times.h>\n#include <sys/types.h>\n
            #include <sys/stat.h>\n#include <sys/resource.h>\n#include <stdlib.h>''','''#define ARRAYSIZE 10000000
            int i,*m;
            struct   rusage temp1,temp2;
            double f0,f1,f2;

            if (getrusage(RUSAGE_SELF,&temp1)) {
              printf("Error calling getrusage()\\n");
              return -1;
            }
            m = malloc(ARRAYSIZE*sizeof(int));
            if (!m) {
              printf("Error calling malloc()\\n");
              return -3;
            }
            for (i=0; i<ARRAYSIZE; i++){
              m[i] = i+1;
            }

            if (getrusage(RUSAGE_SELF,&temp2)) {
              printf("Error calling getrusage()\\n");
              return -1;
            }

            f0 = ((double)(temp2.ru_maxrss-temp1.ru_maxrss))/(4.0*ARRAYSIZE);
            f1 = 1024.0 * ((double)(temp2.ru_maxrss-temp1.ru_maxrss))/(4.0*ARRAYSIZE);
            f2 = getpagesize() * ((double)(temp2.ru_maxrss-temp1.ru_maxrss))/(4.0*ARRAYSIZE);
            printf("Final value %g Initial value %g Increment %g 1K Scaled Increment %g pagesize scaled Increment %g\\n",(double)(temp2.ru_maxrss),(double)(temp1.ru_maxrss),f0,f1,f2);

            if (f1 == 0) {
              printf("getrusage() does not work\\n");
              return 0;
            }
            if (f0 > .90 && f0 < 1.1) {
              printf("uses bytes in getrusage()\\n");
              return 1;
            if (f1 > .90 && f1 < 1.1) {
              printf("uses 1024 size chunks in getrusage()\\n");
              return 2;
            } else if (f2 > .9 && f2 < 1.1) {
              printf("uses getpagesize() chunks in getrusage()\\n");
              return 3;
            }
            printf("unable to determine if uses bytes, 1024 or getpagesize() chunks in getrusage()\\n");
            return -2;''')
        if status > 0:
          if status == 2:
            self.addDefine('USE_KBYTES_FOR_SIZE',1)
          if status == 3:
            self.addDefine('USE_PAGES_FOR_SIZE',1)
        elif status == 0:
          self.delDefine('HAVE_GETRUSAGE')
          self.logPrint("getrusage() does not work (returns 0)")
        else:
          self.delDefine('HAVE_GETRUSAGE')
          self.logPrint("Unable to determine how to use getrusage() memory information")
        self.logPrint("output from getrusage()")
        self.logPrint(output)
        return

      # do not provide a way to get resident set size
      self.delDefine('HAVE_GETRUSAGE')
    return

  def configure(self):
    self.executeTest(self.configureMemorySize)
    return



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

configure_options = [
  '--package-prefix-hash='+petsc_hash_pkgs,
  '--with-cudac=0',
  '--with-debugging=0',
  #'--with-cc=mpicc.openmpi',
  #'--with-cxx=mpicxx.openmpi',
  #'--with-fc=mpif90.openmpi',
  #'--with-mpiexec=mpiexec.openmpi',
  '--download-openmpi=https://download.open-mpi.org/release/open-mpi/v4.1/openmpi-4.1.6.tar.bz2', # way too many CI failures with 5.0.5
  '--download-mpe=1',
  '--download-fblaslapack=1',
  '--download-hypre=1',
  '--download-cmake=1',
  '--download-metis=1',
  '--download-parmetis=1',
  '--download-ptscotch=1',
  '--download-suitesparse=1',
  '--download-triangle=1',
  '--download-triangle-build-exec=1',
  '--download-superlu=1',
  '--download-superlu_dist=1',
  '--download-scalapack=1',
  '--download-mumps=1',
  '--download-elemental=1',
  '--download-spai=1',
  '--download-parms=1',
  '--download-moab=1',
  '--download-chaco=1',
  '--download-fftw=1',
  '--with-petsc4py=1',
  '--download-mpi4py=1',
  '--download-saws',
  '--download-concurrencykit=1',
  '--download-revolve=1',
  '--download-cams=1',
  '--download-p4est=1',
  '--with-zlib=1',
  '--download-mfem=1',
  '--download-glvis=1',
  '--with-opengl=1',
  '--download-libpng=1',
  '--download-libjpeg=1',
  '--download-slepc=1',
  '--download-hpddm=1',
  '--download-bamg=1',
  '--download-mmg=1',
  '--download-parmmg=1',
  '--download-htool=1',
  '--download-egads=1',
  '--download-opencascade=1',
  '--with-strict-petscerrorcode',
  '--with-coverage',
  ]

if __name__ == '__main__':
  import sys,os
  sys.path.insert(0,os.path.abspath('config'))
  import configure
  configure.petsc_configure(configure_options)



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--package-prefix-hash='+petsc_hash_pkgs,
    '--with-make-test-np=20',
    '--with-mpi=0',
    '--with-cc=gcc',
    '--with-cxx=g++',
    '--with-fc=gfortran',
    '--with-cuda=1',
    '--download-hdf5',
    '--download-metis',
    '--download-superlu',
    '--download-mumps',
    '--with-mumps-serial',
    '--download-p4est=1',
    '--with-zlib=1',
    # stress-test h2opus: mpiuni and CPU code while PETSc has GPU support
    '--download-h2opus',
    '--with-cxx-dialect=14',
    '--with-shared-libraries=1',
    '--download-slepc',
    '--download-hpddm',
    '--download-fftw',
    '--with-strict-petscerrorcode',
    '--with-coverage',
  ]
  configure.petsc_configure(configure_options)




import config.package

class Configure(config.package.CMakePackage):
  def __init__(self, framework):
    config.package.CMakePackage.__init__(self, framework)
    self.download          = ['http://eecs.berkeley.edu/~aydin/CombBLAS_FILES/CombBLAS_beta_16_1.tgz']
    self.functions         = ['ParMETIS_V3_PartKway']
    self.includes          = ['parmetis.h']
    self.liblist           = [['libparmetis.a']]
    self.hastests          = 1
    self.buildLanguages    = ['Cxx']
    self.downloaddirnames  = ['CombBLAS']


  def setupDependencies(self, framework):
    config.package.CMakePackage.setupDependencies(self, framework)
    self.compilerFlags = framework.require('config.compilerFlags', self)
    self.mpi           = framework.require('config.packages.MPI',self)
    self.mathlib       = framework.require('config.packages.mathlib',self)
    self.deps          = [self.mpi, self.mathlib]






#!/usr/bin/env python3
from __future__ import print_function
import os
from benchmarkExample import PETScExample

savedTiming = {'baconost': {'ElemAssembly': [(0.040919999999999998, 0.0), (0.1242, 0.0), (0.24410000000000001, 0.0), (0.374, 0.0), (0.56259999999999999, 0.0), (0.79049999999999998, 0.0), (1.0880000000000001, 0.0), (1.351, 0.0), (1.6930000000000001, 0.0), (2.0609999999999999, 0.0), (2.4820000000000002, 0.0), (3.0640000000000001, 0.0)],
                            'MatCUSPSetValBch': [(0.0123, 0.0), (0.023429999999999999, 0.0), (0.043540000000000002, 0.0), (0.06608, 0.0), (0.09579, 0.0), (0.12920000000000001, 0.0), (0.17169999999999999, 0.0), (0.2172, 0.0), (0.27179999999999999, 0.0), (0.48309999999999997, 0.0), (0.44180000000000003, 0.0), (0.51529999999999998, 0.0)]}
               }

def calculateNonzeros(n):
  num = 0
  # corners
  num += 2*3 + 2*4
  # edges
  num += 4*(n-2)*5
  # interior
  num += (n-2)*(n-2)*7
  return num

def processSummary(moduleName, times, events):
  '''Process the Python log summary into plot data'''
  m = __import__(moduleName)
  reload(m)
  # Total Time
  times.append(m.Time[0])
  # Common events
  #   Add the time and flop rate
  for stageName, eventName in [('GPU_Stage','MatCUSPSetValBch'), ('CPU_Stage','ElemAssembly')]:
    s = getattr(m, stageName)
    if not eventName in events:
      events[eventName] = []
    events[eventName].append((s.event[eventName].Time[0], s.event[eventName].Flops[0]/(s.event[eventName].Time[0] * 1e6)))
  return

def plotSummary(library, num, sizes, nonzeros, times, events):
  from pylab import legend, plot, show, title, xlabel, ylabel, ylim
  import numpy as np
  showEventTime      = True
  showTimePerRow     = False
  showTimePerNonzero = True
  print(events)
  if showEventTime:
    data  = []
    names = []
    for event, style in [('MatCUSPSetValBch', 'b-'), ('ElemAssembly', 'b:')]:
      names.append(event)
      data.append(sizes)
      data.append(np.array(events[event])[:,0])
      data.append(style)
    plot(*data)
    title('Performance on '+library+' Example '+str(num))
    xlabel('Number of Dof')
    ylabel('Time (s)')
    legend(names, 'upper left', shadow = True)
    show()
  if showTimePerRow:
    data  = []
    names = []
    for event, style in [('MatCUSPSetValBch', 'b-'), ('ElemAssembly', 'b:')]:
      names.append(event)
      data.append(sizes)
      rows = np.sqrt(sizes)
      data.append(np.array(events[event])[:,0]/rows/3)
      data.append(style)
    plot(*data)
    title('Performance on '+library+' Example '+str(num))
    xlabel('Number of Dof')
    ylabel('Time/Row (s)')
    legend(names, 'upper left', shadow = True)
    show()
  if showTimePerNonzero:
    data  = []
    names = []
    for event, style in [('MatCUSPSetValBch', 'b-'), ('ElemAssembly', 'b:')]:
      names.append(event)
      data.append(sizes)
      data.append(np.array(events[event])[:,0]/nonzeros * 10**9)
      data.append(style)
    plot(*data)
    title('Performance on '+library+' Example '+str(num))
    xlabel('Number of Dof')
    ylabel('Time/Nonzero (ns)')
    legend(names, 'center right', shadow = True)
    show()
  return

if __name__ == '__main__':
  import argparse

  parser = argparse.ArgumentParser(description     = 'PETSc Benchmarking',
                                   epilog          = 'This script runs src/<library>/tutorials/ex<num>, For more information, visit https://petsc.org/',
                                   formatter_class = argparse.ArgumentDefaultsHelpFormatter)
  parser.add_argument('--library', default='SNES',          help='The PETSc library used in this example')
  parser.add_argument('--num',     type = int, default='5', help='The example number')
  parser.add_argument('--module',  default='summary',       help='The module for timing output')
  parser.add_argument('--saved',                            help='Name of saved data')
  parser.add_argument('--scaling',                          help='Run parallel scaling test')
  parser.add_argument('--small',   action='store_true', default=False, help='Use small sizes')
  parser.add_argument('--batch',   action='store_true', default=False, help='Generate batch files for the runs instead')

  args = parser.parse_args()
  print(args)
  ex       = PETScExample(args.library, args.num, log_summary_python = None if args.batch else args.module+'.py', preload='off')
  sizes    = []
  nonzeros = []
  times    = []
  if args.saved is None:
    events   = {}
    if args.scaling == 'strong':
      procs  = [1, 2, 4, 8]
      if args.small:
        grid = [10]*len(procs)
      else:
        grid = [1250]*len(procs)
    else:
      if args.small:
        grid = [100, 150, 200, 250, 300]
      else:
        grid = range(150, 1350, 100)
      procs  = [1]*len(grid)
    for n, p in zip(grid, procs):
      ex.run(p, da_grid_x=n, da_grid_y=n, cusp_synchronize=1, batch=args.batch)
      sizes.append(n*n)
      nonzeros.append(calculateNonzeros(n))
      if not args.batch:
        processSummary(args.module, times, events)
        os.remove(args.module+'.pyc')
  else:
    if args.batch: raise RuntimeException('Cannot use batch option with saved data')
    if args.saved in savedTiming:
      events = savedTiming[args.saved]
    else:
      # Process output to produce module
      events       = {}
      filenameBase = args.saved[:-7]
      jobnumBase   = int(args.saved[-7:])
      for i, n in enumerate(range(150, 1350, 100)):
        filename = filenameBase+str(jobnumBase+i)
        print('Processing',filename)
        headerSeen = False
        with file(filename) as f, file(args.module+'.py', 'w') as o:
          for line in f.readlines():
            if not headerSeen:
              if not line[0] == '#': continue
              headerSeen = True
            if line[0] == '#' and line[-6:] == '=====\n': break
            o.write(line)
            #print line
        processSummary(args.module, times, events)
        # I can't believe that this is necessary
        os.remove(args.module+'.pyc')
    for n in range(150, 1350, 100):
      sizes.append(n*n)
      nonzeros.append(calculateNonzeros(n))
  if not args.batch: plotSummary(args.library, args.num, sizes, nonzeros, times, events)



#!/usr/bin/python3
if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  mpilibdir = os.path.join(os.environ['NMPI_ROOT'],'lib64','ve')
  configure_options = [
    # NEC MPI wrappers (as of version 2.15.0) explicitly list libmpi.a when linking and not -lmpi
    # our checkSharedLinker configure test fails and PETSc will build static libraries
    # uncomment the next two lines if you need PETSc as a shared library
    # '--LDFLAGS=-Wl,-rpath,' + mpilibdir + '-L' + mpilibdir + ' -lmpi',
    # '--with-shared-libraries=1',
    '--with-debugging=0',
    # Need CXX support, and my default installation does not have system g++
    '--download-sowing-configure-arguments=CC=ncc CXX=nc++',
    'PETSC_ARCH=arch-necve',
  ]
  configure.petsc_configure(configure_options)



import config.base

import os

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = 'PETSC'
    return

  def setupDependencies(self, framework):
    config.base.Configure.setupDependencies(self, framework)
    self.setCompilers = framework.require('config.setCompilers', self)
    self.libraries    = framework.require('config.libraries', self)
    return

  def configureCPURelax(self):
    ''' Definitions for cpu relax assembly instructions '''
    # Definition for cpu_relax()
    # From Linux documentation
    # cpu_relax() call can lower power consumption or yield to a hyperthreaded
    # twin processor; it also happens to serve as a compiler barrier

    # x86
    if self.checkCompile('', '__asm__ __volatile__("rep; nop" ::: "memory");'):
      self.addDefine('CPU_RELAX()','__asm__ __volatile__("rep; nop" ::: "memory")')
      return
    # PowerPC
    if self.checkCompile('','do { HMT_low; HMT_medium; __asm__ __volatile__ ("":::"memory"); } while (0)'):
      self.addDefine('CPU_RELAX()','do { HMT_low; HMT_medium; __asm__ __volatile__ ("":::"memory"); } while (0)')
      return
    elif self.checkCompile('','__asm__ __volatile__ ("":::"memory");'):
      self.addDefine('CPU_RELAX()','__asm__ __volatile__ ("":::"memory")')
      return

  # The list is not exhaustive and may not compile on all systems. If you use these macros,
  # you need to have a fallback when the macros are not defined.
  def configureMemoryBarriers(self):
    ''' Definitions for memory barrier instructions'''
    # ---- Definitions for x86_64 -----
    # General Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("mfence":::"memory")'):
      self.addDefine('MEMORY_BARRIER()','__asm__ __volatile__ ("mfence":::"memory")')
    # Read Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("lfence":::"memory")'):
      self.addDefine('READ_MEMORY_BARRIER()','__asm__ __volatile__ ("lfence":::"memory")')
    # Write Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("sfence":::"memory")'):
      self.addDefine('WRITE_MEMORY_BARRIER()','__asm__ __volatile__ ("sfence":::"memory")')

    # ---- Definitions for PowerPC -----
    # General Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("sync":::"memory")'):
      self.addDefine('MEMORY_BARRIER()','__asm__ __volatile__ ("sync":::"memory")')
    # Read Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("lwsync":::"memory")'):
      self.addDefine('READ_MEMORY_BARRIER()','__asm__ __volatile__ ("lwsync":::"memory")')
    # Write Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("eieio":::"memory")'):
      self.addDefine('WRITE_MEMORY_BARRIER()','__asm__ __volatile__ ("eieio":::"memory")')

    # ---- Definitions for ARM v7 -----
    # Use dmb for all memory barrier types
    if self.checkCompile('','__asm__ __volatile__ ("dmb":::"memory")'):
      self.addDefine('MEMORY_BARRIER()','__asm__ __volatile__ ("dmb":::"memory")')
      self.addDefine('READ_MEMORY_BARRIER()','__asm__ __volatile__ ("dmb":::"memory")')
      self.addDefine('WRITE_MEMORY_BARRIER()','__asm__ __volatile__ ("dmb":::"memory")')

    # ---- Definitions for ARM v8 -----
    # Ref: http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.den0024a/CEGDBEJE.html
    # Ref: http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.den0024a/CHDGACJD.html
    # Use "Inner shareable", which makes sense for PETSc.
    # General Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("dmb ish":::"memory")'):
      self.addDefine('MEMORY_BARRIER()','__asm__ __volatile__ ("dmb ish":::"memory")')
    # Read Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("dmb ishld":::"memory")'):
      self.addDefine('READ_MEMORY_BARRIER()','__asm__ __volatile__ ("dmb ishld":::"memory")')
    # Write Memory Barrier
    if self.checkCompile('','__asm__ __volatile__ ("dmb ishst":::"memory")'):
      self.addDefine('WRITE_MEMORY_BARRIER()','__asm__ __volatile__ ("dmb ishst":::"memory")')
    return

  def configure(self):
    # These are not currently used in PETSc
    # self.executeTest(self.configureCPURelax)
    # self.executeTest(self.configureMemoryBarriers)
    return



import config.package

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.gitcommit         = 'dc161a92a1919ca4a7661ee47fc5c89e3860f258' #master dec-1-2018
    self.download          = ['git://https://bitbucket.org/petsc/pkg-radau5.git','https://bitbucket.org/petsc/pkg-radau5/get/'+self.gitcommit+'.tar.gz']
    self.liblist           = [['libradau5.a']]
    self.libDirs           = ['']
    self.precisions        = ['double']
    self.requires32bitint  = 1;
    self.complex           = 0;
    self.buildLanguages    = ['FC'];
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.petscclone     = framework.require('PETSc.options.petscclone',self.setCompilers)
    return

  def Install(self):
    import os
    try:
      self.pushLanguage('FC')
      output,err,ret = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make AR=ar FC=\''+self.getCompiler()+' '+self.getCompilerFlags()+'\'',timeout=2500,log = self.log)
      self.popLanguage()
    except RuntimeError as e:
      raise RuntimeError('Error running make on radau5: '+str(e))
    output,err,ret  = config.package.Package.executeShellCommand('cp -f '+os.path.join(self.packageDir,'libradau5.a')+' '+os.path.join(self.confDir,'lib'), timeout=60, log = self.log)
    return os.path.join(self.confDir,'lib')




#!/usr/bin/env python3
def generateBatchScript(num, procs, time, *args):
  body = '''\
#!/bin/bash
#PBS -N ex%d_GPU_test
#PBS -l walltime=%02d:%02d:%02d
#PBS -l nodes=%d:ppn=1
#PBS -j oe
cd $PBS_O_WORKDIR
echo Main process running on `hostname`
echo Directory is `pwd`
echo PBS has allocated the following nodes:
echo `cat $PBS_NODEFILE`
echo Starting execution at `date +'%a, %d %b %Y %H:%M:%S %z'`
NPROCS=`wc -l < $PBS_NODEFILE`
echo This job has allocated $NPROCS CPUs
# execute an MPI program
echo Executing mpiexec -np $NPROCS ex%d %s
mpiexec -np $NPROCS ex%d %s
''' % (num, (time%86400)/3600, (time%3600)/60, time%60, procs, num, ' '.join(args), num, ' '.join(args))
  namePattern = 'ex%d_%03d.batch'
  for n in range(1000):
    try:
      filename = namePattern % (num, n)
      f = file(filename)
      f.close()
      n += 1
    except IOError as e:
      if e.errno == 2:
        break
      else:
        raise e
  with file(filename, 'w') as f:
    f.write(body)
  return filename

if __name__ == '__main__':
  # Waiting for argparse in 2.7
  import sys
  num   = int(sys.argv[1])
  time  = int(sys.argv[2]) # in seconds
  procs = int(sys.argv[3])
  #args  = ['-da_grid_x 800', '-da_grid_y 800', '-log_view',  '-log_summary_python']
  generateBatchScript(num, procs, time, *sys.argv[4:])



import config.package

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.gitcommit              = '668c2e36991b0a2c39275949277e6ce817414f94'  #master sep-25-2020
    self.download               = ['git://https://github.com/libMesh/libmesh.git','https://github.com/libMesh/libmesh/archive/'+self.gitcommit+'.tar.gz']
    self.functions              = []
    self.includes               = []
    self.skippackagewithoptions = 1
    self.useddirectly           = 0
    self.linkedbypetsc          = 0
    self.builtafterpetsc        = 1
    return

  def setupHelp(self, help):
    config.package.Package.setupHelp(self,help)
    import nargs
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.setCompilers    = framework.require('config.setCompilers',self)
    self.sharedLibraries = framework.require('PETSc.options.sharedLibraries', self)
    self.installdir      = framework.require('PETSc.options.installDir',self)
    return

  def Install(self):
    import os
    # if installing prefix location then need to set new value for PETSC_DIR/PETSC_ARCH
    if self.argDB['prefix']:
       newdir = 'PETSC_DIR='+os.path.abspath(os.path.expanduser(self.argDB['prefix']))+' '
       prefix = os.path.abspath(os.path.expanduser(self.argDB['prefix']))
    else:
       newdir = ' '
       prefix = os.path.join(self.petscdir.dir,self.arch)

    self.addDefine('HAVE_LIBMESH',1)
    self.addMakeMacro('LIBMESH','yes')
    self.addMakeRule('libmeshbuild','', \
                       ['@echo "*** Building libmesh ***"',\
                          '@${RM} ${PETSC_ARCH}/lib/petsc/conf/libmesh.errorflg',\
                          '@(cd '+self.packageDir+' && \\\n\
           '+newdir+' ./configure --prefix='+prefix+' && \\\n\
           '+newdir+' '+self.make.make_jnp+' ) > ${PETSC_ARCH}/lib/petsc/conf/libmesh.log 2>&1 || \\\n\
             (echo "**************************ERROR*************************************" && \\\n\
             echo "Error building libmesh. Check ${PETSC_ARCH}/lib/petsc/conf/libmesh.log" && \\\n\
             echo "********************************************************************" && \\\n\
             touch ${PETSC_ARCH}/lib/petsc/conf/libmesh.errorflg && \\\n\
             exit 1)'])
    self.addMakeRule('libmeshinstall','', \
                       ['@echo "*** Installing libmesh ***"',\
                          '@(cd '+self.packageDir+' && \\\n\
           '+newdir+' make install ) >> ${PETSC_ARCH}/lib/petsc/conf/libmesh.log 2>&1 || \\\n\
             (echo "**************************ERROR*************************************" && \\\n\
             echo "Error installing libmesh. Check ${PETSC_ARCH}/lib/petsc/conf/libmesh.log" && \\\n\
             echo "********************************************************************" && \\\n\
             exit 1)'])
    if self.argDB['prefix']:
      self.addMakeRule('libmesh-build','')
      # the build must be done at install time because PETSc shared libraries must be in final location before building libmesh
      self.addMakeRule('libmesh-install','libmeshbuild libmeshinstall')
    else:
      self.addMakeRule('libmesh-build','libmeshbuild libmeshinstall')
      self.addMakeRule('libmesh-install','')

    return self.installDir

  def alternateConfigureLibrary(self):
    self.addMakeRule('libmesh-build','')
    self.addMakeRule('libmesh-install','')




import config.package
import os

class Configure(config.package.GNUPackage):
  def __init__(self, framework):
    config.package.GNUPackage.__init__(self, framework)
    self.version           = '2.8.6'
    self.gitcommit         = 'v'+self.version
    self.download          = ['git://https://github.com/cburstedde/p4est','https://github.com/cburstedde/p4est/archive/'+self.gitcommit+'.tar.gz']
    self.versionname       = 'P4EST_VERSION_MAJOR.P4EST_VERSION_MINOR.P4EST_VERSION_POINT'
    self.versioninclude    = 'p4est_config.h'
    self.functions         = ['p4est_init']
    self.includes          = ['p4est_bits.h']
    self.liblist           = [['libp4est.a', 'libsc.a']]
    self.downloadonWindows = 1
    return

  def setupHelp(self,help):
    '''Default GNU setupHelp, but p4est debugging option'''
    config.package.GNUPackage.setupHelp(self,help)
    import nargs
    help.addArgument(self.PACKAGE,'-with-p4est-debugging=<bool>',nargs.ArgBool(None,0,"Use p4est's (sometimes computationally intensive) debugging"))
    return

  def setupDependencies(self, framework):
    config.package.GNUPackage.setupDependencies(self, framework)
    self.mpi        = framework.require('config.packages.MPI',self)
    self.blasLapack = framework.require('config.packages.BlasLapack',self)
    self.zlib       = framework.require('config.packages.zlib',self)
    self.memalign   = framework.argDB['with-memalign']
    self.deps       = [self.blasLapack,self.zlib]
    self.odeps      = [self.mpi]
    return

  def formGNUConfigureArgs(self):
    from shlex import quote
    args = config.package.GNUPackage.formGNUConfigureArgs(self)
    if self.argDB['with-p4est-debugging']:
      args.append('--enable-debug')
    if not self.mpi.usingMPIUni:
      args.append('--enable-mpi')
      if self.mpi.mpiexecExecutable:
        args.append('PATH='+quote(os.environ['PATH']+':'+os.path.dirname(self.mpi.mpiexecExecutable)))
    else:
      args.append('--disable-mpi')
    args.append('CPPFLAGS='+quote(self.headers.toStringNoDupes(self.dinclude)))
    args.append('LIBS='+quote(self.libraries.toString(self.dlib)))
    args.append('--enable-memalign='+self.memalign)
    return args

  def updateGitDir(self):
    config.package.GNUPackage.updateGitDir(self)
    if not hasattr(self.sourceControl, 'git') or (self.packageDir != os.path.join(self.externalPackagesDir,'git.'+self.package)):
      return
    Dir = self.getDir()
    try:
      libsc = self.libsc
    except AttributeError:
      try:
        self.executeShellCommand([self.sourceControl.git, 'submodule', 'update', '--init'], cwd=Dir, log=self.log)
        if os.path.isfile(os.path.join(Dir,'sc','README')):
          self.libsc = os.path.join(Dir,'sc')
        else:
          raise RuntimeError
      except RuntimeError:
        raise RuntimeError('Could not initialize sc submodule needed by p4est')
    return

  def preInstall(self):
    self.Bootstrap('./bootstrap')



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

configure_options = [
  '--package-prefix-hash='+petsc_hash_pkgs,
  '--with-cc=gcc -m32',
  '--with-cxx=g++ -m32',
  '--with-fc=gfortran -m32',

  '--with-64-bit-indices=1',
  '--with-log=0',
  '--with-info=0',
  '--with-ctable=0',
  '--with-is-color-value-type=short',
  '--with-single-library=0',
  '--with-strict-petscerrorcode',

  '--with-c2html=0',
  '--with-x=0',
  '--download-mpich',
  '--download-fblaslapack',
  ]

if __name__ == '__main__':
  import sys,os
  sys.path.insert(0,os.path.abspath('config'))
  import configure
  configure.petsc_configure(configure_options)



import config.package
import os

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.download         = ['http://software.lanl.gov/ascem/tpls/ascem-io-2.2.tar.gz']
    self.downloaddirnames = ['ascem-io']
    self.includes         = ['ascemio_util.h']
    self.liblist          = [['libparallelio.a']]
    self.functions        = []
    self.hastests         = 1
    self.useddirectly     = 0
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.compilerFlags   = framework.require('config.compilerFlags', self)
    self.mpi             = framework.require('config.packages.MPI',self)
    self.hdf5            = framework.require('config.packages.hdf5',self)
    self.deps            = [self.mpi,self.hdf5]
    return

  def Install(self):
    import os
    self.pushLanguage('C')
    if self.hdf5.include:
      MAKEARGS = 'MACHINE="" CC="'+self.getCompiler()+' '+self.updatePackageCFlags(self.getCompilerFlags())+'" HDF5_INCLUDE_DIR="'+self.hdf5.include[0]+'"'
    else:
      MAKEARGS = 'MACHINE="" CC="'+self.getCompiler()+' '+self.updatePackageCFlags(self.getCompilerFlags())+'"'
    self.popLanguage()
    INSTALLARGS = 'ASCEMIO_INSTALL_DIR="'+self.installDir+'"'
    g = open(os.path.join(self.packageDir,'compiledata'),'w')
    g.write(MAKEARGS+INSTALLARGS)
    g.close()

    if self.installNeeded('compiledata'):
      try:
        self.logPrintBox('Compiling and installing ascem-io; this may take several minutes')
        output,err,ret = config.package.Package.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && '+self.make.make+' '+MAKEARGS+' && '+self.make.make+' '+INSTALLARGS+' install', timeout=2500, log = self.log)
      except RuntimeError as e:
        raise RuntimeError('Error running make on ascem-io: '+str(e))
      self.postInstall(output+err,'compiledata')
    return self.installDir





#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--package-prefix-hash='+petsc_hash_pkgs,
    #'--with-mpi-dir=/home/users/balay/soft/instinct/gcc-10.2.0/mpich-4.1',
    #'--with-blaslapack-dir=/home/users/balay/soft/instinct/gcc-10.2.0/fblaslapack',
    '--with-make-np=24',
    '--with-make-test-np=8',
    '--with-hipc=/opt/rocm-5.4.3/bin/hipcc',
    '--with-hip-dir=/opt/rocm-5.4.3',
    'COPTFLAGS=-g -O',
    'FOPTFLAGS=-g -O',
    'CXXOPTFLAGS=-g -O',
    'HIPOPTFLAGS=-g -O',
    '--with-cuda=0',
    '--with-hip=1',
    '--with-precision=double',
    '--with-clanguage=c',
    '--download-kokkos',
    '--download-kokkos-kernels',
    '--download-hypre',
    '--download-magma',
    '--with-magma-fortran-bindings=0',
    '--download-mfem',
    '--download-metis',
    '--with-strict-petscerrorcode',
    #'--with-coverage',
  ]

  configure.petsc_configure(configure_options)



import config.package

class Configure(config.package.GNUPackage):
  def __init__(self, framework):
    config.package.GNUPackage.__init__(self, framework)
    self.version          = '2.5.0'
    self.maxversion       = '2.5.100000'
    self.versionname      = 'SUNDIALS_PACKAGE_VERSION'
    self.download         = ['https://web.cels.anl.gov/projects/petsc/download/externalpackages/sundials-'+self.version+'p1.tar.gz']
    self.downloaddirnames = ['sundials']
    self.functions        = ['CVSpgmr','CVDense']
    self.includes         = ['sundials/sundials_nvector.h']
    self.liblist          = [['libsundials_cvode.a','libsundials_nvecserial.a','libsundials_nvecparallel.a']] #currently only support CVODE
    self.parallelMake     = 0  # uses recursive make so better be safe and not use make -j np
    self.complex          = 0
    self.precisions       = ['double']
    self.hastests         = 1

  def setupDependencies(self, framework):
    config.package.GNUPackage.setupDependencies(self, framework)
    self.blasLapack = framework.require('config.packages.BlasLapack',self)
    self.mpi        = framework.require('config.packages.MPI',self)
    self.mathlib    = framework.require('config.packages.mathlib',self)
    self.deps       = [self.mpi,self.blasLapack,self.mathlib]

  def formGNUConfigureArgs(self):
    import os
    args = config.package.GNUPackage.formGNUConfigureArgs(self)

    self.logPrintBox('Installing SUNDIALS 2.5 (April 2012) which is a very old version of SUNDIALS. PETSc does not directly interface with newer versions of SUNDIALS.')
    self.pushLanguage('C')
    # use --with-mpi-root if we know it works
    if self.mpi.directory and (os.path.realpath(self.getCompiler())).find(os.path.realpath(self.mpi.directory)) >=0:
      self.log.write('Sundials configure: using --with-mpi-root='+self.mpi.directory+'\n')
      args.append('--with-mpi-root="'+self.mpi.directory+'"')
    # else provide everything!
    else:
      #print a message if the previous check failed
      if self.mpi.directory:
        self.log.write('Sundials configure: --with-mpi-dir specified - but could not use it\n')
        self.log.write(str(os.path.realpath(self.getCompiler()))+' '+str(os.path.realpath(self.mpi.directory))+'\n')

      if self.mpi.include:
        args.append('--with-mpi-incdir="'+self.mpi.include[0]+'"')
      else:
        args.append('--with-mpi-incdir="/usr/include"')  # dummy case

      if self.mpi.lib:
        args.append('--with-mpi-libdir="'+os.path.dirname(self.mpi.lib[0])+'"')
        libs = []
        for l in self.mpi.lib:
          ll = os.path.basename(l)
          if ll.endswith('.a'): libs.append(ll[3:-2])
          elif ll.endswith('.so'): libs.append(ll[3:-3])
          elif ll.endswith('.dylib'): libs.append(ll[3:-6])
          libs.append(ll[3:-2])
        libs = '-l' + ' -l'.join(libs)
        args.append('--with-mpi-libs="'+libs+'"')
      else:
        args.append('--with-mpi-libdir="/usr/lib"')  # dummy case
        args.append('--with-mpi-libs="-lc"')

    self.popLanguage()

    args = self.rmArgsStartsWith(args,['F77=','F90=','FC=','FFLAGS=','FCFLAGS=','F90FLAGS='])
    args.append('--without-mpif77')
    args.append('--disable-examples')
    args.append('--disable-cvodes')
    args.append('--disable-ida')
    args.append('--disable-idas')
    args.append('--disable-cpodes')
    args.append('--disable-fcmix')
    args.append('--disable-kinsol')

    args.append('--disable-f77') #does not work? Use 'F77=no' instead
    args.append('F77=no')
    args.append('--disable-libtool-lock')
    return [arg for arg in args if not arg in ['--enable-shared']]



#!/usr/bin/python3

# Follow instructions at https://www.alcf.anl.gov/support-center/aurorasunspot/getting-started-aurora
# to set up the proxy settings in your .bashrc and git with SSH protocol in your .ssh/config

# module use /soft/modulefiles
# module load spack-pe-oneapi cmake python
# module load  oneapi/eng-compiler/2023.10.15.002
#
# Currently Loaded Modules:
# 1) gcc/11.2.0            5) spack-pe-gcc/0.5-rc1         9) mpich/52.2-256/icc-all-pmix-gpu
# 2) libfabric/1.15.2.0    6) spack-pe-oneapi/0.5-rc1     10) intel_compute_runtime/release/agama-devel-682.22
# 3) cray-pals/1.2.12      7) cmake/3.26.4-gcc-testing    11) oneapi/eng-compiler/2023.10.15.002
# 4) cray-libpals/1.2.12   8) python/3.10.10-gcc-testing

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--with-cc=mpicc',
    '--with-cxx=mpicxx',
    '--with-fc=mpifort',
    '--with-debugging=0',
    '--with-mpiexec-tail=gpu_tile_compact.sh',
    '--SYCLPPFLAGS=-Wno-tautological-constant-compare',
    '--with-sycl',
    '--with-syclc=icpx',
    '--with-sycl-arch=pvc',
    '--COPTFLAGS=-O2 -g',
    '--FOPTFLAGS=-O2 -g',
    '--CXXOPTFLAGS=-O2 -g',
    '--SYCLOPTFLAGS=-O2 -g',
    '--download-kokkos',
    '--download-kokkos-kernels',
  ]
  configure.petsc_configure(configure_options)



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

oadirf='"/cygdrive/c/Program Files (x86)/Intel/oneAPI"'
oadir=os.popen('cygpath -u '+os.popen('cygpath -ms '+oadirf).read()).read().strip()
oamkldir=oadir+'/mkl/2022.1.0/lib/intel64'
oampidir=oadir+'/mpi/2021.6.0'

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--package-prefix-hash='+petsc_hash_pkgs,
    '--with-debugging=0',
    '--with-blaslapack-lib=-L'+oamkldir+' mkl_intel_lp64_dll.lib mkl_sequential_dll.lib mkl_core_dll.lib',
    '--with-cc=cl',
    '--with-cxx=cl',
    '--with-fc=ifort',
    '--with-mpi-include='+oampidir+'/include',
    '--with-mpi-lib='+oampidir+'/lib/release/impi.lib',
    '--with-mpiexec='+oampidir+'/bin/mpiexec -localonly',
    '--download-metis',
    '--download-parmetis',
    '--with-strict-petscerrorcode=0',
  ]
  configure.petsc_configure(configure_options)



from __future__ import generators
import config.base

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = ''
    self.substPrefix  = ''
    return

  def __str1__(self):
    if not hasattr(self, 'memalign'):
      return ''
    return '  Memory alignment from malloc(): ' + self.memalign + ' bytes\n'

  def setupHelp(self, help):
    import nargs
    help.addArgument('PETSc', '-with-memalign=<4,8,16,32,64>', nargs.Arg(None, '16', 'Specify alignment of arrays allocated by PETSc'))
    return

  def setupDependencies(self, framework):
    config.base.Configure.setupDependencies(self, framework)
    self.types     = framework.require('config.types', self)
    self.languages = framework.require('PETSc.options.languages', self)
    self.compilers = framework.require('config.compilers', self)
    return

  def configureMemAlign(self):
    '''Choose memory alignment'''
    # Intel/AMD cache lines are 64 bytes, default page sizes are usually 4kB. It would be pretty silly to want that much alignment by default.
    valid = ['4', '8', '16', '32', '64', '128', '256', '512', '1024', '2048', '4096', '8192']
    self.memalign = self.framework.argDB['with-memalign']
    if self.memalign in valid:
      self.addDefine('MEMALIGN', self.memalign)
    else:
      raise RuntimeError('--with-memalign must be in' + str(valid))
    self.logPrint('Memory alignment is ' + self.memalign)
    return

  def configure(self):
    self.executeTest(self.configureMemAlign)
    return



import config.package


class Configure(config.package.CMakePackage):
  def __init__(self, framework):
    config.package.CMakePackage.__init__(self, framework)
    self.version                = '0.4.1'
    self.gitcommit              = '0.4.1'
    self.download               = ['git://https://github.com/limix/hcephes.git','https://github.com/limix/hcephes/archive/refs/tags/'+self.gitcommit+'.tar.gz']
    self.includes               = ['hcephes.h']
    self.hastests               = 1
    self.functions              = ['hcephes_erf']
    self.liblist                = [['libhcephes.a']]
    self.precisions             = ['double']
    self.useddirectly           = 0
  def setupDependencies(self, framework):
    config.package.CMakePackage.setupDependencies(self, framework)
    self.compilerFlags = framework.require('config.compilerFlags', self)
    self.mathlib       = framework.require('config.packages.mathlib', self)
    self.deps          = [self.mathlib]



import config.package
import os

class Configure(config.package.CMakePackage):
  def __init__(self, framework):
    config.package.CMakePackage.__init__(self, framework)
    self.version          = '2.2.2'
    self.gitcommit        = '0e8767285b7a201c7b1ff34d2c2bb009534145df' # master (CMake 4.0.0 fix) apr-08-2025
    self.download         = ['git://https://github.com/Reference-ScaLAPACK/scalapack','https://github.com/Reference-ScaLAPACK/scalapack/archive/'+self.gitcommit+'.tar.gz']
    self.includes         = []
    self.liblist          = [['libscalapack.a'],
                             ['libmkl_scalapack_lp64.a','libmkl_blacs_intelmpi_lp64.a'],
                             ['libmkl_scalapack_lp64.a','libmkl_blacs_mpich_lp64.a'],
                             ['libmkl_scalapack_lp64.a','libmkl_blacs_sgimpt_lp64.a'],
                             ['libmkl_scalapack_lp64.a','libmkl_blacs_openmpi_lp64.a']]
    self.functions        = ['pssytrd']
    self.functionsFortran = 1
    self.buildLanguages   = ['FC']
    self.precisions       = ['single','double']
    self.downloadonWindows= 1
    self.makerulename     = 'scalapack'
    self.minCmakeVersion  = (3,26,0)
    self.libDirs          = ['lib',os.path.join('lib','intel64')]
    self.requirekandr     = 1
    self.brokengnu23      = 1
    return

  def setupDependencies(self, framework):
    config.package.CMakePackage.setupDependencies(self, framework)
    self.flibs      = framework.require('config.packages.flibs',self)
    self.blasLapack = framework.require('config.packages.BlasLapack',self)
    self.mpi        = framework.require('config.packages.MPI',self)
    self.deps       = [self.mpi, self.blasLapack, self.flibs]
    return

  def formCMakeConfigureArgs(self):
    args = config.package.CMakePackage.formCMakeConfigureArgs(self)
    args.append('-DLAPACK_LIBRARIES="'+self.libraries.toString(self.blasLapack.dlib)+'"')
    args.append('-DSCALAPACK_BUILD_TESTS=OFF')
    return args

def getSearchDirectories(self):
  '''Generate list of possible locations of ScaLAPACK'''
  yield ''
  if 'with-'+self.package+'-dir' in self.argDB:
    d = self.argDB['with-'+self.package+'-dir']
    for libdir in self.libDirs:
      yield os.path.join(d,libdir)
  if os.getenv('MKLROOT'):
    for libdir in self.libDirs:
      yield os.path.join(os.getenv('MKLROOT'),libdir)



from mpi4py import MPI
import numpy

comm = MPI.COMM_WORLD
rank = comm.Get_rank()

# passing MPI datatypes explicitly
if rank == 0:
    data = numpy.arange(1000, dtype='i')
    comm.Send([data, MPI.INT], dest=1, tag=77)
elif rank == 1:
    data = numpy.empty(1000, dtype='i')
    comm.Recv([data, MPI.INT], source=0, tag=77)

# automatic MPI datatype discovery
if rank == 0:
    data = numpy.arange(100, dtype=numpy.float64)
    comm.Send(data, dest=1, tag=13)
elif rank == 1:
    data = numpy.empty(100, dtype=numpy.float64)
    comm.Recv(data, source=0, tag=13)



import config.package
import os

class Configure(config.package.Package):
  def __init__(self, framework):
    config.package.Package.__init__(self, framework)
    self.includes             = ['mkl_pardiso.h']
    self.functions            = ['pardisoinit']
    self.liblist              = [[]] # use MKL detected by BlasLapack.py
    self.precisions           = ['single','double']
    self.hastests             = 1
    self.requires32bitintblas = 0
    return

  def setupDependencies(self, framework):
    config.package.Package.setupDependencies(self, framework)
    self.blasLapack = framework.require('config.packages.BlasLapack',self)
    self.deps       = [self.blasLapack]
    return

  def consistencyChecks(self):
    config.package.Package.consistencyChecks(self)
    if self.argDB['with-'+self.package]:
      if not self.blasLapack.mkl:
        raise RuntimeError('MKL PARDISO requires Intel MKL. Please rerun configure using --with-blaslapack-dir=LOCATION_OF_INTEL_MKL')
      elif self.blasLapack.has64bitindices and not self.defaultIndexSize == 64:
        raise RuntimeError('MKL PARDISO cannot work with 32-bit integers but 64-bit BLAS/LAPACK integers')
    return

  def configureLibrary(self):
    config.package.Package.configureLibrary(self)
    self.usesopenmp = self.blasLapack.usesopenmp



#!/usr/bin/env python3

import os
petsc_hash_pkgs=os.path.join(os.getenv('HOME'),'petsc-hash-pkgs')

configure_options = [
  '--package-prefix-hash='+petsc_hash_pkgs,
  '--with-debugging=0',
  'CC=gcc',
  'CXX=g++',
  'FC=gfortran',
  '--with-mpi-include=/usr/include/x86_64-linux-gnu/mpich',
  '--with-mpi-lib=-L/usr/lib/x86_64-linux-gnu -lmpichfort -lmpich',
  '--download-f2cblaslapack=1',
  '--download-slepc=1',
  '--download-hpddm=1',
  '--with-precision=__float128',
  '--with-clanguage=cxx',
  '--with-mpi-f90module-visibility=0',
  '--with-strict-petscerrorcode',
  '--with-coverage',
  ]

if __name__ == '__main__':
  import sys,os
  sys.path.insert(0,os.path.abspath('config'))
  import configure
  configure.petsc_configure(configure_options)



import unittest
from petsc4py import PETSc
import os
from PetscBinaryIO import *

class TestPetscBinaryIO(unittest.TestCase):
    def setUp(self):
        try:
            os.remove('test.dat')
        except OSError:
            pass
        try:
            os.remove('test.dat.info')
        except OSError:
            pass

    def test_VecRead(self):
        """Test reading a Vec"""
        array = np.array([1.1, 2.2, 3.3])
        vec = PETSc.Vec().createSeq(3)
        vec[...] = array
        viewer = PETSc.Viewer().createBinary('test.dat', PETSc.Viewer.Mode.W)
        vec.view(viewer)
        viewer.destroy()
        vec.destroy()

        result, = PetscBinaryIO().readBinaryFile('test.dat')
        self.assertTrue(np.allclose(array, result))

    def test_VecWrite(self):
        """Test writing a Vec"""
        array = np.array([1.1, 2.2, 3.3])
        PetscBinaryIO().writeBinaryFile('test.dat', [array.view(Vec),])

        vec = PETSc.Vec().createSeq(3)
        vec.set(0.)
        viewer = PETSc.Viewer().createBinary('test.dat', PETSc.Viewer.Mode.R)
        vec.load(viewer)
        viewer.destroy()

        self.assertTrue(np.allclose(array, vec[...]))
        vec.destroy()

    def test_ISRead(self):
        """Test reading an IS"""
        indices = np.array([3,4,5])
        anis = PETSc.IS().createGeneral(list(indices))
        viewer = PETSc.Viewer().createBinary('test.dat', PETSc.Viewer.Mode.W)
        anis.view(viewer)
        viewer.destroy()
        anis.destroy()

        result, = PetscBinaryIO().readBinaryFile('test.dat')
        self.assertTrue((indices == result).all())

    def test_MatRead(self):
        """Test reading a Mat"""
        mat = PETSc.Mat().createAIJ(2)
        mat[0,0] = 1.1
        mat[0,1] = 2.1
        mat[1,1] = 3.1
        mat.assemble()

        vals = np.array([1.1,2.1,3.1])
        counts = np.array([0,2,3])
        cols = np.array([0,1,1])

        viewer = PETSc.Viewer().createBinary('test.dat', PETSc.Viewer.Mode.W)
        mat.view(viewer)
        viewer.destroy()
        mat.destroy()

        result, = PetscBinaryIO().readBinaryFile('test.dat')
        self.assertTrue(np.allclose(vals, result[1][2]))
        self.assertTrue((counts == result[1][0]).all())
        self.assertTrue((cols == result[1][1]).all())
        self.assertTrue((2,2) == result[0])

    def test_MatWrite(self):
        """Test writing a Mat"""
        vals = np.array([1.1,2.1,3.1])
        counts = np.array([0,2,3])
        cols = np.array([0,1,1])
        mat = MatSparse(((2,2),(counts,cols,vals)))

        dense = np.array([1.1,2.1,0.0,3.1])

        PetscBinaryIO().writeBinaryFile('test.dat', [mat,])

        mat = PETSc.Mat().createAIJ(2)
        viewer = PETSc.Viewer().createBinary('test.dat', PETSc.Viewer.Mode.R)
        mat.load(viewer)
        viewer.destroy()

        self.assertTrue(np.allclose(dense, mat[:,:].ravel()))
        mat.destroy()


if __name__ == '__main__':
    unittest.main()
    try:
        os.remove('test.dat')
    except OSError:
        pass
    try:
        os.remove('test.dat.info')
    except OSError:
        pass



import config.package

class Configure(config.package.GNUPackage):
  def __init__(self, framework):
    config.package.GNUPackage.__init__(self, framework)
    self.gitcommit              = 'f38dd83e30136b4e25eb2343813ee4fbd7c16681'
    #self.gitcommit              = '5.2.0'
    self.download               = ['git://https://github.com/flame/libflame.git','https://github.com/flame/libflame/archive/'+self.gitcommit+'.tar.gz']
    self.downloaddirnames       = ['libflame']
    self.functions              = ['FLA_Cntl_gemv_obj_create']
    self.includes               = ['FLAME.h']
    self.liblist                = [['libflame.a','libblis.a'],['libflame.a','libblis-mt.a']]
    self.precisions             = ['single','double']
    self.buildLanguages         = ['C','FC']
    return

  def setupDependencies(self, framework):
    config.package.GNUPackage.setupDependencies(self, framework)
    self.compilerFlags = framework.require('config.compilerFlags', self)
    self.blis          = framework.require('config.packages.blis',self)
    self.deps          = [self.blis]
    return

  def formGNUConfigureArgs(self):
    args = config.package.GNUPackage.formGNUConfigureArgs(self)
    args.append('--enable-lapack2flame')
    args.append('--disable-warnings')
    args.append('--enable-max-arg-list-hack')
    if self.argDB['with-shared-libraries']:
      args.append('--enable-dynamic-build')
    return [arg for arg in args if not arg in ['--enable-shared']]

  def Install(self):
    import os
    args = self.formGNUConfigureArgs()
    if self.download and self.argDB['download-'+self.downloadname.lower()+'-configure-arguments']:
       args.append(self.argDB['download-'+self.downloadname.lower()+'-configure-arguments'])
    args = ' '.join(args)
    conffile = os.path.join(self.packageDir,self.package+'.petscconf')
    fd = open(conffile, 'w')
    fd.write(args)
    fd.close()
    ### Use conffile to check whether a reconfigure/rebuild is required
    if not self.installNeeded(conffile):
      return self.installDir
    self.preInstall()

    ### Configure and Build package
    try:
      self.logPrintBox('Running configure on ' +self.PACKAGE+'; this may take several minutes')
      output1,err1,ret1  = config.base.Configure.executeShellCommand(os.path.join('.',self.configureName)+' '+args, cwd=self.packageDir, timeout=2000, log = self.log)
    except RuntimeError as e:
      self.logPrint('Error running configure on ' + self.PACKAGE+': '+str(e))
      try:
        with open(os.path.join(self.packageDir,'config.log')) as fd:
          conf = fd.read()
          fd.close()
          self.logPrint('Output in config.log for ' + self.PACKAGE+': '+conf)
      except:
        pass
      raise RuntimeError('Error running configure on ' + self.PACKAGE)

    # Patch main Makefile so that FFLAGS is taken into account
    fname = os.path.join(self.packageDir,'Makefile')
    oldcode = r'''$(FC) -cpp -c $$< -o $$@'''
    newcode = r'''$(FC) $(FFLAGS) -cpp -c $$< -o $$@'''
    with open(fname,'r') as file:
      sourcecode = file.read()
    sourcecode = sourcecode.replace(oldcode,newcode)
    with open(fname,'w') as file:
      file.write(sourcecode)

    try:
      self.logPrintBox('Running make on '+self.PACKAGE+'; this may take several minutes')
      if self.parallelMake: pmake = self.make.make_jnp+' '+self.makerulename+' '
      else: pmake = self.make.make+' '+self.makerulename+' '

      output2,err2,ret2  = config.base.Configure.executeShellCommand(self.make.make+' clean', cwd=self.packageDir, timeout=200, log = self.log)
      output3,err3,ret3  = config.base.Configure.executeShellCommand(pmake+' V=1 all', cwd=self.packageDir, timeout=6000, log = self.log)
      self.logPrintBox('Running make install on '+self.PACKAGE+'; this may take several minutes')
      output4,err4,ret4  = config.base.Configure.executeShellCommand(self.make.make+' install', cwd=self.packageDir, timeout=1000, log = self.log)
    except RuntimeError as e:
      self.logPrint('Error running make; make install on '+self.PACKAGE+': '+str(e))
      raise RuntimeError('Error running make; make install on '+self.PACKAGE)
    self.postInstall(output1+err1+output2+err2+output3+err3+output4+err4, conffile)
    return self.installDir



import config.base
import os
import re

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    self.headerPrefix = 'PETSC'
    self.substPrefix  = 'PETSC'
    return

  def __str1__(self):
    if hasattr(self, 'dir'):
      return '  PETSC_DIR: '+str(self.dir)+'\n'
    return ''

  def setupHelp(self, help):
    import nargs
    help.addArgument('PETSc', '-PETSC_DIR=<root-dir>',                        nargs.Arg(None, None, 'The root directory of the PETSc installation'))
    return

  def configureDirectories(self):
    '''Checks PETSC_DIR and sets if not set'''
    if 'PETSC_DIR' in self.framework.argDB:
      self.dir = os.path.normpath(self.framework.argDB['PETSC_DIR'])
      msg1 = 'The configure option'
      msg2 = ''
    elif 'PETSC_DIR' in os.environ:
      self.dir = os.path.normpath(os.environ['PETSC_DIR'])
      msg1 = 'The environmental variable'
      msg2 = 'export'
    else:
      self.dir = os.getcwd()
      msg1 = ''
      msg2 = ''

    if self.dir == 'pwd':
      raise RuntimeError('{0} PETSC_DIR=pwd is incorrect. You need to use back quotes around the pwd - i.e: {1} PETSC_DIR=`pwd`'.format(msg1, msg2))
    elif self.dir.find(' ') > -1:
      raise RuntimeError('{0} PETSC_DIR="{1}" has spaces in it; this is not allowed. Change the directory with PETSc to not have spaces in it'.format(msg1, self.dir))
    elif not os.path.isabs(self.dir):
      raise RuntimeError('{0} PETSC_DIR={1} is a relative path. Use absolute path - i.e: {2} PETSC_DIR={3}'.format(msg1, self.dir, msg2, os.path.abspath(self.dir)))
    elif not os.path.isdir(self.dir):
      raise RuntimeError('{0} PETSC_DIR={1} is not a directory'.format(msg1, self.dir))
    elif os.path.realpath(self.dir) != os.path.realpath(os.getcwd()):
      raise RuntimeError('{0} PETSC_DIR={1} MUST be the current directory {2}'.format(msg1, self.dir, os.getcwd()))

    self.version  = 'Unknown'
    versionHeader = os.path.join(self.dir, 'include', 'petscversion.h')
    versionInfo = []
    if os.path.exists(versionHeader):
      f = open(versionHeader)
      for line in f:
        if line.find('define PETSC_VERSION') >= 0:
          versionInfo.append(line[:-1])
      f.close()
    else:
      raise RuntimeError('Invalid PETSc directory '+str(self.dir)+'. Could not locate '+versionHeader)
    self.versionRelease = True if versionInfo[0].split(' ')[-1] == '1' else False
    if self.versionRelease:
      self.version = '.'.join([line.split(' ')[-1] for line in versionInfo[1:4]])
    else:
      self.version = '.'.join([line.split(' ')[-1] for line in versionInfo[1:3]])
      self.version += '.99'
    self.logPrint('PETSC_VERSION_RELEASE of 1 indicates the code is from a release branch or a branch created from a release branch.')
    self.logPrint('Version Information:')
    for line in versionInfo:
      self.logPrint(line)
    dirs = self.argDB['with-executables-search-path']
    if not isinstance(dirs, list): dirs = dirs.split(os.path.pathsep)
    dirs.append(os.path.join(self.dir, 'lib','petsc','bin', 'win32fe'))
    self.framework.argDB['with-executables-search-path'] = dirs
    return

  def configure(self):
    self.executeTest(self.configureDirectories)
    return



#!/usr/bin/env python3

# Example configure script for Perlmutter, the HPE Cray EX system at NERSC/LBNL equipped with
# AMD EPYC CPUS and NVIDIA A100 GPUS. Here we target the GPU compute nodes and builds with
# support for the CUDA/cuSPARSE, Kokkos, and ViennaCL back-ends.
#
# Currently, configuring PETSc on the system does not require loading many , if any, non-default modules.
# As documented at https://docs.nersc.gov/systems/perlmutter/software/#mpi, typical settings might be
#
#   export MPICH_GPU_SUPPORT_ENABLED=1
#   module load cudatoolkit
#   module load PrgEnv-gnu
#   module load craype-accel-nvidia80
#   module load cray-python
#
# The above are currently present in the default environment. Users may wish to 'module load' a
# different programming environment (which will generally force a reload of certain related modules,
# such as the one corresponding to the MPI implementation).

if __name__ == '__main__':
  import sys
  import os
  sys.path.insert(0, os.path.abspath('config'))
  import configure
  configure_options = [
    '--with-make-np=8', # Must limit size of parallel build to stay within resource limitations imposed by the center
    # '-G4' requests all four GPUs present on a Perlmutter GPU compute node.
    # --gpu-bind=none to avoid the gpu-aware mpi runtime error: (GTL DEBUG: 0) cuIpcOpenMemHandle: invalid argument, CUDA_ERROR_INVALID_VALUE, line no 360
    '--with-mpiexec=srun -G4 --gpu-bind=none',
    '--with-batch=0',

    # Use the Cray compiler wrappers, regardless of the underlying compilers loaded by the programming environment module:
    '--with-cc=cc',
    '--with-cxx=CC',
    '--with-fc=ftn',

    # Build with aggressive optimization ('-O3') but also include debugging symbols ('-g') to support detailed profiling.
    # If you are doing development, using no optimization ('-O0') can be a good idea. Also note that some compilers (GNU
    # is one) support the '-g3' debug flag, which allows macro expansion in some debuggers; this can be very useful when
    # debugging PETSc code, as PETSc makes extensive use of macros.
    '--COPTFLAGS=   -g -O3',
    '--CXXOPTFLAGS= -g -O3',
    '--FOPTFLAGS=   -g -O3',
    '--CUDAFLAGS=   -g -O3',
    '--with-debugging=0',  # Disable debugging for production builds; use '--with-debugging=1' for development work.

    # Build with support for CUDA/cuSPARSE, Kokkos/Kokkos Kernels, and ViennaCL back-ends:
    '--with-cuda=1',
    '--with-cuda-arch=80',
    '--download-viennacl',
    '--download-kokkos',
    '--download-kokkos-kernels',

    # Download and build a few commonly-used packages:
    '--download-hypre',
    '--download-metis',
    '--download-parmetis',
    '--download-hdf5', # Note that NERSC does provide an HDF5 module, but using our own is generally reliable.
    '--download-hdf5-fortran-bindings',
  ]
  configure.petsc_configure(configure_options)
